@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2002, 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,32 +15,35 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.c,v 1.27.2.3 2003/12/18 12:48:42 chpe Exp $
+ *  $Id: ephy-history.c,v 1.40 2003/11/09 00:45:30 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "ephy-types.h"
 #include "ephy-history.h"
 #include "ephy-file-helpers.h"
-#include "ephy-autocompletion-source.h"
 #include "ephy-debug.h"
+#include "ephy-node-db.h"
 #include "ephy-node-common.h"
 
 #include <time.h>
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
+#define EPHY_HISTORY_XML_ROOT	 "ephy_history"
 #define EPHY_HISTORY_XML_VERSION "1.0"
 
 /* how often to save the history, in milliseconds */
-#define HISTORY_SAVE_INTERVAL (60 * 5 * 1000)
+#define HISTORY_SAVE_INTERVAL (5 * 60 * 1000)
 
+/* if you change this remember to change also the user interface description */
 #define HISTORY_PAGE_OBSOLETE_DAYS 10
 
+#define EPHY_HISTORY_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_HISTORY, EphyHistoryPrivate))
+
 struct EphyHistoryPrivate
 {
 	char *xml_file;
@@ -49,9 +52,12 @@ struct EphyHistoryPrivate
 	EphyNode *pages;
 	EphyNode *last_page;
 	GHashTable *hosts_hash;
+	GStaticRWLock *hosts_hash_lock;
 	GHashTable *pages_hash;
+	GStaticRWLock *pages_hash_lock;
 	int autosave_timeout;
 	guint update_hosts_idle;
+	gboolean dirty;
 };
 
 enum
@@ -69,8 +75,6 @@ static void
 ephy_history_init (EphyHistory *tab);
 static void
 ephy_history_finalize (GObject *object);
-static void
-ephy_history_autocompletion_source_init (EphyAutocompletionSourceIface *iface);
 
 static GObjectClass *parent_class = NULL;
 
@@ -96,85 +100,14 @@ ephy_history_get_type (void)
                         (GInstanceInitFunc) ephy_history_init
                 };
 
-		static const GInterfaceInfo autocompletion_source_info =
-		{
-			(GInterfaceInitFunc) ephy_history_autocompletion_source_init,
-			NULL,
-			NULL
-		};
-
                 ephy_history_type = g_type_register_static (G_TYPE_OBJECT,
 							      "EphyHistory",
 							      &our_info, 0);
-
-		g_type_add_interface_static (ephy_history_type,
-					     EPHY_TYPE_AUTOCOMPLETION_SOURCE,
-					     &autocompletion_source_info);
         }
 
         return ephy_history_type;
 }
 
-static void
-ephy_history_autocompletion_source_set_basic_key (EphyAutocompletionSource *source,
-						  const gchar *basic_key)
-{
-	/* nothing to do here */
-}
-
-static void
-ephy_history_autocompletion_source_foreach (EphyAutocompletionSource *source,
-					    const gchar *current_text,
-					    EphyAutocompletionSourceForeachFunc func,
-					    gpointer data)
-{
-	GPtrArray *children;
-	int i;
-	EphyHistory *eb = EPHY_HISTORY (source);
-	GTime now;
-
-	now = time (NULL);
-
-	children = ephy_node_get_children (eb->priv->pages);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-		const char *url, *title;
-		int last_visit, visits;
-		guint32 score;
-
-		kid = g_ptr_array_index (children, i);
-		g_assert (kid != NULL);
-
-		url = ephy_node_get_property_string
-			(kid, EPHY_NODE_PAGE_PROP_LOCATION);
-		title = ephy_node_get_property_string
-			(kid, EPHY_NODE_PAGE_PROP_TITLE);
-		last_visit = ephy_node_get_property_int
-			(kid, EPHY_NODE_PAGE_PROP_LAST_VISIT);
-		visits = ephy_node_get_property_int
-			(kid, EPHY_NODE_PAGE_PROP_VISITS);
-		score = MAX (visits - ((now - last_visit) >> 15), 1);
-
-		func (source, url,
-		      url, url, FALSE,
-		      FALSE, score, data);
-	}
-}
-
-static void
-ephy_history_emit_data_changed (EphyHistory *eb)
-{
-	g_signal_emit_by_name (eb, "data-changed");
-}
-
-static void
-ephy_history_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
-{
-	iface->foreach = ephy_history_autocompletion_source_foreach;
-	iface->set_basic_key = ephy_history_autocompletion_source_set_basic_key;
-}
-
 static void
 ephy_history_class_init (EphyHistoryClass *klass)
 {
@@ -194,34 +127,8 @@ ephy_history_class_init (EphyHistoryClass *klass)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
-}
 
-static void
-ephy_history_load (EphyHistory *eb)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-
-	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
-		return;
-
-	doc = xmlParseFile (eb->priv->xml_file);
-	if (doc == NULL)
-	{
-		g_warning ("Failed to load history from %s!\n", eb->priv->xml_file);
-		return;
-	}
-
-	root = xmlDocGetRootElement (doc);
-
-	for (child = root->children; child != NULL; child = child->next)
-	{
-		EphyNode *node;
-
-		node = ephy_node_new_from_xml (eb->priv->db, child);
-	}
-
-	xmlFreeDoc (doc);
+	g_type_class_add_private (object_class, sizeof(EphyHistoryPrivate));
 }
 
 static gboolean
@@ -253,6 +160,7 @@ remove_obsolete_pages (EphyHistory *eb)
         g_date_set_time (&current_date, time (NULL));
 
 	children = ephy_node_get_children (eb->priv->pages);
+	ephy_node_thaw (eb->priv->pages);
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
@@ -269,44 +177,31 @@ remove_obsolete_pages (EphyHistory *eb)
 static void
 ephy_history_save (EphyHistory *eb)
 {
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	GPtrArray *children;
-	int i;
+	int ret;
 
-	LOG ("Saving history")
-
-	/* save nodes to xml */
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
-
-	root = xmlNewDocNode (doc, NULL, "ephy_history", NULL);
-	xmlSetProp (root, "version", EPHY_HISTORY_XML_VERSION);
-	xmlDocSetRootElement (doc, root);
-
-	children = ephy_node_get_children (eb->priv->hosts);
-	for (i = 0; i < children->len; i++)
+	/* only save if there are changes */
+	if (eb->priv->dirty == FALSE)
 	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-		if (kid == eb->priv->pages) continue;
-
-		ephy_node_save_to_xml (kid, root);
+		return;
 	}
 
-	children = ephy_node_get_children (eb->priv->pages);
-	for (i = 0; i < children->len; i++)
+	LOG ("Saving history db")
+
+	ret = ephy_node_db_write_to_xml_safe
+		(eb->priv->db, eb->priv->xml_file,
+		 EPHY_HISTORY_XML_ROOT,
+		 EPHY_HISTORY_XML_VERSION,
+		 NULL, /* comment */
+		 eb->priv->hosts,
+		 1, eb->priv->pages,
+		 eb->priv->pages, 0,
+		 NULL);
+
+	if (ret >=0)
 	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_save_to_xml (kid, root);
+		/* save was successful */
+		eb->priv->dirty = FALSE;
 	}
-
-	ephy_file_save_xml (eb->priv->xml_file, doc);
-	xmlFreeDoc(doc);
 }
 
 static void
@@ -314,9 +209,15 @@ hosts_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
+	eb->priv->dirty = TRUE;
+
+	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
+
 	g_hash_table_insert (eb->priv->hosts_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
+
+	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
 }
 
 static void
@@ -325,8 +226,22 @@ hosts_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
+	eb->priv->dirty = TRUE;
+
+	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
+
 	g_hash_table_remove (eb->priv->hosts_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
+
+	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
+}
+
+static void
+hosts_changed_cb (EphyNode *node,
+		  EphyNode *child,
+		  EphyHistory *eb)
+{
+	eb->priv->dirty = TRUE;
 }
 
 static void
@@ -334,9 +249,15 @@ pages_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
+	eb->priv->dirty = TRUE;
+
+	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
+
 	g_hash_table_insert (eb->priv->pages_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
+
+	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
 }
 
 static void
@@ -345,8 +266,22 @@ pages_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
+	eb->priv->dirty = TRUE;
+
+	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
+
 	g_hash_table_remove (eb->priv->pages_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
+
+	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
+}
+
+static void
+pages_changed_cb (EphyNode *node,
+		  EphyNode *child,
+		  EphyHistory *eb)
+{
+	eb->priv->dirty = TRUE;
 }
 
 static gboolean
@@ -383,6 +318,7 @@ update_host_on_child_remove (EphyNode *node)
 			new_host_last_visit = last_visit;
 		}
 	}
+	ephy_node_thaw (node);
 
 	if (host_last_visit != new_host_last_visit)
 	{
@@ -422,6 +358,7 @@ update_hosts (EphyHistory *eh)
 			}
 		}
 	}
+	ephy_node_thaw (eh->priv->hosts);
 
 	g_list_foreach (empty, (GFunc)ephy_node_unref, NULL);
 	g_list_free (empty);
@@ -463,8 +400,7 @@ ephy_history_init (EphyHistory *eb)
 	GValue value = { 0, };
 	EphyNodeDb *db;
 
-        eb->priv = g_new0 (EphyHistoryPrivate, 1);
-
+        eb->priv = EPHY_HISTORY_GET_PRIVATE (eb);
 	eb->priv->update_hosts_idle = 0;
 
 	db = ephy_node_db_new (EPHY_NODE_DB_HISTORY);
@@ -476,8 +412,13 @@ ephy_history_init (EphyHistory *eb)
 
 	eb->priv->pages_hash = g_hash_table_new (g_str_hash,
 			                          g_str_equal);
+	eb->priv->pages_hash_lock = g_new0 (GStaticRWLock, 1);
+	g_static_rw_lock_init (eb->priv->pages_hash_lock);
+
 	eb->priv->hosts_hash = g_hash_table_new (g_str_hash,
 			                         g_str_equal);
+	eb->priv->hosts_hash_lock = g_new0 (GStaticRWLock, 1);
+	g_static_rw_lock_init (eb->priv->hosts_hash_lock);
 
 	/* Pages */
 	eb->priv->pages = ephy_node_new_with_id (db, PAGES_NODE_ID);
@@ -505,6 +446,10 @@ ephy_history_init (EphyHistory *eb)
 					 EPHY_NODE_CHILD_REMOVED,
 				         (EphyNodeCallback) pages_removed_cb,
 					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->pages,
+					 EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) pages_changed_cb,
+					 G_OBJECT (eb));
 
 	/* Hosts */
 	eb->priv->hosts = ephy_node_new_with_id (db, HOSTS_NODE_ID);
@@ -517,16 +462,24 @@ ephy_history_init (EphyHistory *eb)
 					 EPHY_NODE_CHILD_REMOVED,
 				         (EphyNodeCallback) hosts_removed_cb,
 					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->hosts,
+					 EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) hosts_changed_cb,
+					 G_OBJECT (eb));
 
 	ephy_node_add_child (eb->priv->hosts, eb->priv->pages);
 
-	ephy_history_load (eb);
-	ephy_history_emit_data_changed (eb);
+	ephy_node_db_load_from_file (eb->priv->db, eb->priv->xml_file,
+				     EPHY_HISTORY_XML_ROOT,
+				     EPHY_HISTORY_XML_VERSION);
 
 	g_hash_table_foreach (eb->priv->hosts_hash,
 			      (GHFunc) connect_page_removed_from_host,
 			      eb);
 
+	/* mark as clean */
+	eb->priv->dirty = FALSE;
+
 	/* setup the periodic history saving callback */
 	eb->priv->autosave_timeout =
 		g_timeout_add (HISTORY_SAVE_INTERVAL,
@@ -537,13 +490,7 @@ ephy_history_init (EphyHistory *eb)
 static void
 ephy_history_finalize (GObject *object)
 {
-        EphyHistory *eb;
-
-	g_return_if_fail (IS_EPHY_HISTORY (object));
-
-	eb = EPHY_HISTORY (object);
-
-        g_return_if_fail (eb->priv != NULL);
+        EphyHistory *eb = EPHY_HISTORY (object);
 
 	if (eb->priv->update_hosts_idle)
 	{
@@ -558,12 +505,12 @@ ephy_history_finalize (GObject *object)
 	g_object_unref (eb->priv->db);
 
 	g_hash_table_destroy (eb->priv->pages_hash);
+	g_static_rw_lock_free (eb->priv->pages_hash_lock);
 	g_hash_table_destroy (eb->priv->hosts_hash);
+	g_static_rw_lock_free (eb->priv->hosts_hash_lock);
 
 	g_source_remove (eb->priv->autosave_timeout);
 
-        g_free (eb->priv);
-
 	LOG ("Global history finalized");
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -572,11 +519,11 @@ ephy_history_finalize (GObject *object)
 EphyHistory *
 ephy_history_new ()
 {
-	EphyHistory *tab;
+	EphyHistory *eh;
 
-	tab = EPHY_HISTORY (g_object_new (EPHY_HISTORY_TYPE, NULL));
+	eh = EPHY_HISTORY (g_object_new (EPHY_TYPE_HISTORY, NULL));
 
-	return tab;
+	return eh;
 }
 
 static void
@@ -609,20 +556,29 @@ ephy_history_host_visited (EphyHistory *eh,
 
 static EphyNode *
 ephy_history_add_host (EphyHistory *eh, EphyNode *page)
+{
+	const char *url;
+
+	url = ephy_node_get_property_string
+		(page, EPHY_NODE_PAGE_PROP_LOCATION);
+
+	return ephy_history_get_host (eh, url);
+}
+
+EphyNode *
+ephy_history_get_host (EphyHistory *eh, const char *url)
 {
 	GnomeVFSURI *vfs_uri = NULL;
 	EphyNode *host = NULL;
 	const char *host_name = NULL;
 	GList *host_locations = NULL, *l;
 	GValue value = { 0, };
-	const char *url;
 	const char *scheme = NULL;
 	GTime now;
 
-	now = time (NULL);
+	g_return_val_if_fail (url != NULL, NULL);
 
-	url = ephy_node_get_property_string
-		(page, EPHY_NODE_PAGE_PROP_LOCATION);
+	now = time (NULL);
 
 	vfs_uri = gnome_vfs_uri_new (url);
 
@@ -670,12 +626,15 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 
 	g_return_val_if_fail (host_locations != NULL, NULL);
 
+	g_static_rw_lock_reader_lock (eh->priv->hosts_hash_lock);
+
 	for (l = host_locations; l != NULL; l = l->next)
 	{
 		host = g_hash_table_lookup (eh->priv->hosts_hash,
 					    (char *)l->data);
 		if (host) break;
 	}
+	g_static_rw_lock_reader_unlock (eh->priv->hosts_hash_lock);
 
 	if (!host)
 	{
@@ -769,7 +728,6 @@ ephy_history_visited (EphyHistory *eh, EphyNode *node)
 	eh->priv->last_page = node;
 
 	g_signal_emit (G_OBJECT (eh), ephy_history_signals[VISITED], 0, url);
-	ephy_history_emit_data_changed (eh);
 }
 
 int
@@ -834,7 +792,9 @@ ephy_history_get_page (EphyHistory *eb,
 {
 	EphyNode *node;
 
+	g_static_rw_lock_reader_lock (eb->priv->pages_hash_lock);
 	node = g_hash_table_lookup (eb->priv->pages_hash, url);
+	g_static_rw_lock_reader_unlock (eb->priv->pages_hash_lock);
 
 	return node;
 }
