@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-history.h,v 1.9 2003/11/19 19:19:55 chpe Exp $
  */
 
 #ifndef EPHY_HISTORY_H
@@ -45,13 +47,16 @@ enum
 	EPHY_NODE_PAGE_PROP_FIRST_VISIT = 6,
 	EPHY_NODE_PAGE_PROP_HOST_ID = 7,
 	EPHY_NODE_PAGE_PROP_PRIORITY = 8,
-	EPHY_NODE_PAGE_PROP_ICON = 9
+	EPHY_NODE_PAGE_PROP_ICON = 9,
+	EPHY_NODE_HOST_PROP_ZOOM = 10
 };
 
 struct EphyHistory
 {
-        GObject parent;
-        EphyHistoryPrivate *priv;
+	GObject parent;
+
+	/*< private >*/
+	EphyHistoryPrivate *priv;
 };
 
 struct EphyHistoryClass
@@ -67,6 +72,9 @@ EphyHistory    *ephy_history_new		(void);
 
 EphyNode       *ephy_history_get_hosts          (EphyHistory *gh);
 
+EphyNode       *ephy_history_get_host		(EphyHistory *gh,
+						 const char *url);
+
 EphyNode       *ephy_history_get_pages          (EphyHistory *gh);
 
 EphyNode       *ephy_history_get_page           (EphyHistory *gh,
