@@ -28,8 +28,6 @@ NS_IMPL_ISUPPORTS2(MozGlobalHistory, nsIGlobalHistory, nsIBrowserHistory)
 
 MozGlobalHistory::MozGlobalHistory ()
 {
-	NS_INIT_ISUPPORTS();
-
 	mGlobalHistory = ephy_embed_shell_get_global_history (embed_shell);
 }
 
@@ -90,6 +88,13 @@ NS_IMETHODIMP MozGlobalHistory::GetLastPageVisited(char **aLastPageVisited)
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+#if MOZILLA_SNAPSHOT > 8
+NS_IMETHODIMP MozGlobalHistory::SetLastPageVisited(const char *aLastPageVisited)
+{
+        return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
 NS_IMETHODIMP MozGlobalHistory::HidePage(const char *url)
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
@@ -102,22 +107,6 @@ NS_IMETHODIMP MozGlobalHistory::GetCount(PRUint32 *aCount)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-#if MOZILLA_SNAPSHOT <= 7
- 
-/* void startBatchUpdate (); */
-NS_IMETHODIMP MozGlobalHistory::StartBatchUpdate()
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-/* void endBatchUpdate (); */
-NS_IMETHODIMP MozGlobalHistory::EndBatchUpdate()
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-#endif
-
 /* void markPageAsTyped (in string url); */
 NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(const char *url)
 {
