@@ -44,7 +44,9 @@
 #include "nsIWebProgressListener.h"
 #include "nsIHelperAppLauncherDialog.h"
 #include "nsIExternalHelperAppService.h"
+#include "nsIDOMDocument.h"
 
+#include "nsString.h"
 #include "nsIURI.h"
 #include "nsILocalFile.h"
 #include "nsIWebBrowserPersist.h"
@@ -74,6 +76,12 @@
 
 #define MOZ_DOWNLOAD_CLASSNAME "Ephy's Download Progress Dialog"
 
+nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceUri,
+				  nsILocalFile* inDestFile, const char *contentType,
+				  nsIURI* inOriginalURI, MozillaEmbedPersist *embedPersist,
+				  nsIInputStream *postData, nsISupports *aCacheKey);
+nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval);
+
 class MozDownload : public nsIDownload,
                     public nsIWebProgressListener
 {
