@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2001,2002,2003 Philip Langdale
+ *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,10 +15,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: MozRegisterComponents.cpp,v 1.19 2003/11/26 18:43:05 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ContentHandler.h"
@@ -28,15 +31,19 @@
 #include "MozDownload.h"
 #include "ExternalProtocolService.h"
 #include "EphyAboutRedirector.h"
+#include "EphyContentPolicy.h"
 
 #ifdef HAVE_MOZILLA_PSM
+#include "GtkNSSClientAuthDialogs.h"
 #include "GtkNSSDialogs.h"
+#include "GtkNSSKeyPairDialogs.h"
 #endif
 
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsCOMPtr.h>
 #include <nsILocalFile.h>
+#include <nsNetCID.h>
 
 #include <glib.h>
 
@@ -49,19 +56,28 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GNewsProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(EphyContentPolicy)
+
+#if MOZILLA_SNAPSHOT < 12
 NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
+#endif
 
 #ifdef HAVE_MOZILLA_PSM
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSKeyPairDialogs)
 #endif
 
 static const nsModuleComponentInfo sAppComps[] = {
+#if MOZILLA_SNAPSHOT < 12
 	{
 		G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
 		G_EXTERNALPROTOCOLSERVICE_CID,
 		NS_EXTERNALPROTOCOLSERVICE_CONTRACTID,
  		GExternalProtocolServiceConstructor
 	},
+#endif
 	{
 		MOZ_DOWNLOAD_CLASSNAME,
 		MOZ_DOWNLOAD_CID,
@@ -75,12 +91,30 @@ static const nsModuleComponentInfo sAppComps[] = {
 		GFilePickerConstructor
 	},
 #ifdef HAVE_MOZILLA_PSM
+	{
+		GTK_NSSCLIENTAUTHDIALOGS_CLASSNAME,
+		GTK_NSSCLIENTAUTHDIALOGS_CID,
+		NS_CLIENTAUTHDIALOGS_CONTRACTID,
+		GtkNSSClientAuthDialogsConstructor
+	},
 	{
 		GTK_NSSDIALOGS_CLASSNAME,
 		GTK_NSSDIALOGS_CID,
 		NS_BADCERTLISTENER_CONTRACTID,
 		GtkNSSDialogsConstructor
 	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_CERTIFICATEDIALOGS_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
+	{
+		GTK_NSSKEYPAIRDIALOGS_CLASSNAME,
+		GTK_NSSKEYPAIRDIALOGS_CID,
+		NS_GENERATINGKEYPAIRINFODIALOGS_CONTRACTID,
+		GtkNSSKeyPairDialogsConstructor
+	},
 #endif
 	{
 		NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
@@ -153,21 +187,7 @@ static const nsModuleComponentInfo sAppComps[] = {
 		EPHY_ABOUT_REDIRECTOR_CID,
 		EPHY_ABOUT_REDIRECTOR_MARCO_CONTRACTID,
 		EphyAboutRedirectorConstructor
-	}
-};
-
-static const int sNumAppComps = sizeof(sAppComps) / sizeof(nsModuleComponentInfo);
-
-static const nsModuleComponentInfo sFtpComps = {
-	G_FTP_PROTOCOL_CLASSNAME,
-	G_FTP_PROTOCOL_CID,
-	G_FTP_PROTOCOL_CONTRACTID,
-	GFtpProtocolHandlerConstructor
-};
-
-NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
-
-static const nsModuleComponentInfo sMailtoComps[] = {
+	},
 	{
 		G_MAILTO_PROTOCOL_CLASSNAME,
 		G_MAILTO_PROTOCOL_CID,
@@ -179,17 +199,22 @@ static const nsModuleComponentInfo sMailtoComps[] = {
 		G_MAILTO_PROTOCOL_CID,
 		G_MAILTO_CONTENT_CONTRACTID,
 		GMailtoProtocolHandlerConstructor
+	},
+	{
+		EPHY_CONTENT_POLICY_CLASSNAME,
+		EPHY_CONTENT_POLICY_CID,
+		EPHY_CONTENT_POLICY_CONTRACTID,
+		EphyContentPolicyConstructor
 	}
 };
 
-static const int sNumMailtoComps = sizeof(sMailtoComps) / sizeof(nsModuleComponentInfo);
+static const int sNumAppComps = sizeof(sAppComps) / sizeof(nsModuleComponentInfo);
 
-static const nsModuleComponentInfo sModuleComps[] = {
-	{
-		 G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
-		 G_EXTERNALPROTOCOLSERVICE_CID,
-		 NS_EXTERNALPROTOCOLSERVICE_CONTRACTID
-	}
+static const nsModuleComponentInfo sFtpComps = {
+	G_FTP_PROTOCOL_CLASSNAME,
+	G_FTP_PROTOCOL_CID,
+	G_FTP_PROTOCOL_CONTRACTID,
+	GFtpProtocolHandlerConstructor
 };
 
 static NS_DEFINE_CID(knsFtpProtocolHandlerCID, NS_FTPPROTOCOLHANDLER_CID);
@@ -290,37 +315,3 @@ mozilla_unregister_FtpProtocolHandler (void)
 	ftpRegistered = PR_FALSE;
         return NS_SUCCEEDED (rv) ? TRUE : FALSE;
 }
-
-/**
- * mozilla_register_MailtoProtocolHandler: Register Mailto Protocol Handler
- */
-gboolean 
-mozilla_register_MailtoProtocolHandler (void)
-{
-	gboolean retVal = TRUE;
-        nsresult rv;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	for (int i = 0; i < sNumMailtoComps; i++)
-	{
-		nsCOMPtr<nsIGenericFactory> componentFactory;
-		rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
-					  &(sMailtoComps[i]));
-		if (NS_FAILED(rv))
-		{
-			retVal = FALSE;
-			continue;  // don't abort registering other components
-		}
-
-		rv = cr->RegisterFactory(sMailtoComps[i].mCID,
-					 sMailtoComps[i].mDescription,
-					 sMailtoComps[i].mContractID,
-					 componentFactory);
-		if (NS_FAILED(rv))
-			retVal = FALSE;
-	}
-	return retVal;
-}
