@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2000 Nate Case 
+ *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,10 +15,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: mozilla-notifiers.cpp,v 1.39 2003/12/02 11:41:09 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ephy-embed-shell.h"
@@ -33,13 +36,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <stdlib.h>
-#include "nsCOMPtr.h"
-#include "nsIPrefService.h"
-#include "nsIServiceManager.h"
+#include <gdk/gdkx.h>
+#include <nsCOMPtr.h>
+#include <nsIPrefService.h>
+#include <nsIServiceManager.h>
+#include <libgnome/gnome-i18n.h>
 
 #define MOZILLA_PREF_NO_PROXIES "network.proxy.no_proxies_on"
+#define MIGRATE_PIXEL_SIZE
 
 static void
 mozilla_cache_size_notifier (GConfClient *client,
@@ -348,7 +354,7 @@ mozilla_proxy_autoconfig_notifier (GConfClient *client,
 
 	url = eel_gconf_get_string (entry->key);
 
-	if (url)
+	if (url && url[0] != '\0')
 	{
 		ephy_embed_single_load_proxy_autoconf (single, url);
 	}
@@ -382,14 +388,85 @@ add_notification_and_notify (GConfClient		*client,
 	gconf_client_notify (client, key);
 }
 
+#ifdef MIGRATE_PIXEL_SIZE
+
+#define INT_ROUND(a) gint((a) + 0.5f)
+
+/**
+ *  This function gets the dpi in the same way that mozilla gets the dpi, 
+ *  this allows us to convert from pixels to points easily
+ */
+static gint
+mozilla_get_dpi ()
+{
+	GtkSettings* settings = gtk_settings_get_default ();
+	gint dpi = 0;
+ 
+	/* Use the gdk-xft-dpi setting if it is set */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (G_OBJECT (settings)),
+					  "gtk-xft-dpi"))
+	{
+		g_object_get (G_OBJECT (settings), "gtk-xft-dpi", &dpi, NULL);
+		if (dpi) return INT_ROUND (dpi / PANGO_SCALE);
+	}
+
+	/* Fall back to what xft thinks it is */
+	char *val = XGetDefault (GDK_DISPLAY (), "Xft", "dpi");
+	if (val)
+	{
+		char *e;
+		double d = strtod(val, &e);
+		if (e != val) return INT_ROUND (d);
+	}
+	
+	/* Fall back to calculating manually from the gdk screen settings */
+	float screenWidthIn = float (gdk_screen_width_mm()) / 25.4f;
+	return INT_ROUND (gdk_screen_width() / screenWidthIn);
+}
+
+static void
+mozilla_migrate_font_gconf_key (const char *pixel_key, const char *point_key)
+{
+	int size;
+
+	size = eel_gconf_get_integer (pixel_key);
+
+	if (size > 0)
+	{
+		/* Use doubles to get more accurate arithmetic */
+		double dpi   = (double)mozilla_get_dpi ();
+		double value = (double)eel_gconf_get_integer (pixel_key);
+		gint point = INT_ROUND ((value * 72) / dpi);
+
+		eel_gconf_set_integer (point_key, point);
+	}
+}
+
+#endif
+
 void 
-mozilla_notifiers_init(EphyEmbedSingle *single) 
+mozilla_notifiers_init (EphyEmbedSingle *single) 
 {
 	GConfClient *client = eel_gconf_client_get_global ();
 	guint i;
 	const EphyFontsLanguageInfo *font_languages;
 	guint n_font_languages;
 
+	eel_gconf_monitor_add ("/apps/epiphany/web");
+	eel_gconf_monitor_add ("/system/proxy");
+	eel_gconf_monitor_add ("/system/http_proxy");
+
+#ifdef MIGRATE_PIXEL_SIZE
+	gboolean migrate_size;
+
+	migrate_size = (eel_gconf_get_integer (CONF_SCHEMA_VERSION)
+			< EPIPHANY_SCHEMA_VERSION);
+	if (migrate_size)
+	{
+		eel_gconf_set_integer (CONF_SCHEMA_VERSION, EPIPHANY_SCHEMA_VERSION);
+	}
+#endif
+
 	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
 	{
 		GConfClientNotifyFunc func = NULL;
@@ -436,6 +513,9 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 		char *types [] = { "variable", "monospace" };
 		char key[255];
 		char *info;
+#ifdef MIGRATE_PIXEL_SIZE
+		char old_key[255];
+#endif
 		
 		for (k = 0; k < G_N_ELEMENTS (types); k++)
 		{
@@ -449,6 +529,34 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 			font_infos = g_list_prepend (font_infos, info);
 		}
 
+#ifdef MIGRATE_PIXEL_SIZE
+                if (migrate_size)
+                {
+			char *type;
+
+			type = eel_gconf_get_string (CONF_RENDERING_FONT_TYPE_OLD);
+			if (type && (strcmp (type, "serif") == 0 ||
+				     strcmp (type, "sans-serif") == 0))
+			{
+				char *family;
+
+				g_snprintf (old_key, 255, "%s_%s_%s",
+					    CONF_RENDERING_FONT, type, code);
+				g_snprintf (key, 255, "%s_%s_%s",
+					    CONF_RENDERING_FONT, "variable", code);
+
+				family = eel_gconf_get_string (old_key);
+				if (family)
+				{
+					eel_gconf_set_string (key, family);
+					g_free (family);
+				}
+			}
+
+			g_free (type);
+		}
+#endif
+
 		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, code);
 		info = g_strconcat ("minimum-size", ".", code, NULL);
 		add_notification_and_notify (client, key,
@@ -456,6 +564,15 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 					     info);
 		font_infos = g_list_prepend (font_infos, info);
 
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_MIN_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
+
 		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, code);
 		info = g_strconcat ("size.fixed", ".", code, NULL);
 		add_notification_and_notify (client, key,
@@ -463,19 +580,44 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 					     info);
 		font_infos = g_list_prepend (font_infos, info);
 
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_FIXED_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
+
 		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, code);
 		info = g_strconcat ("size.variable", ".", code, NULL);
 		add_notification_and_notify (client, key,
 					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
 					     info);
-		font_infos = g_list_prepend (font_infos, info);		
+		font_infos = g_list_prepend (font_infos, info);
+
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_VAR_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
 	}
 }
 
 void 
 mozilla_notifiers_free (void)
 {
-	ephy_notification_remove (&mozilla_notifiers);
+	eel_gconf_monitor_remove ("/apps/epiphany/web");
+	eel_gconf_monitor_remove ("/system/proxy");
+	eel_gconf_monitor_remove ("/system/http_proxy");
+
+	g_list_foreach (mozilla_notifiers, 
+		        (GFunc)eel_gconf_notification_remove, 
+		        NULL);
+	g_list_free(mozilla_notifiers);
 
 	g_list_foreach (font_infos, (GFunc) g_free, NULL);
 	g_list_free (font_infos);
