@@ -431,20 +431,20 @@ parse_toolbars (EggToolbarsModel *t,
     }
 }
 
-void
+gboolean
 egg_toolbars_model_load (EggToolbarsModel *t,
 			 const char *xml_file)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), FALSE);
 
   doc = xmlParseFile (xml_file);
   if (doc == NULL)
   {
     g_warning ("Failed to load XML data from %s", xml_file);
-    return;
+    return FALSE;
   }
   root = xmlDocGetRootElement (doc);
 
@@ -452,6 +452,8 @@ egg_toolbars_model_load (EggToolbarsModel *t,
   parse_toolbars (t, root->children);
 
   xmlFreeDoc (doc);
+
+  return TRUE;
 }
 
 static char *
