@@ -25,7 +25,7 @@
 
 #include <ctype.h>
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-help.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkmessagedialog.h>
@@ -113,46 +113,3 @@ ephy_gui_help (GtkWindow *parent,
 		g_error_free (err);
 	}
 }
-
-#define ICON_NAME_MIME_PREFIX "gnome-mime-"
-
-static char *
-make_mime_name (const char *mime_type)
-{
-	char *mime_type_without_slashes, *icon_name;
-	char *p;
-                                                                                                                              
-	if (mime_type == NULL)
-	{
-		return NULL;
-	}
-                                                                                                                              
-	mime_type_without_slashes = g_strdup (mime_type);
-                                                                                                                              
-	while ((p = strchr(mime_type_without_slashes, '/')) != NULL)
-		*p = '-';
-                                                                                                                              
-	icon_name = g_strconcat (ICON_NAME_MIME_PREFIX, mime_type_without_slashes, NULL);
-	g_free (mime_type_without_slashes);
-                                                                                                                              
-	return icon_name;
-}
-
-GdkPixbuf *
-ephy_gui_get_pixbuf_from_mime_type (const char *mime_type,
-                                    int size)
-{
-	GdkPixbuf *pixbuf;
-	GtkIconTheme *icon_theme;
-	char *icon_name;
-
-	icon_name = make_mime_name (mime_type);
-
-	icon_theme = gtk_icon_theme_get_default ();
-	g_return_val_if_fail (icon_theme != NULL, NULL);
-
-	pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name,
-					   size, 0, NULL);
-
-	return pixbuf;
-}
