@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000 Marco Pesenti Gritti
+ *  Copyright (C) 2000, 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persc
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,70 +15,26 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-langs.h,v 1.7 2003/10/17 22:40:15 chpe Exp $
  */
 
 #ifndef EPHY_LANGS_H
 #define EPHY_LANGS_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <glib.h>
 
 G_BEGIN_DECLS
 
-/* language groups */
-typedef enum
-{
-	LG_ARABIC,
-	LG_BALTIC,
-	LG_CENTRAL_EUROPEAN,
-	LG_CHINESE,
-	LG_CYRILLIC,
-	LG_GREEK,
-	LG_HEBREW,
-	LG_INDIAN,
-	LG_JAPANESE,
-	LG_KOREAN,
-	LG_TURKISH,
-	LG_UNICODE,
-	LG_VIETNAMESE,
-	LG_WESTERN,
-	LG_OTHER,
-	LG_ALL
-} LanguageGroup;
-
 typedef struct
 {
-	gchar *title;
-	gchar *key;
-	LanguageGroup group;
-} LanguageGroupInfo;
+	char *title;
+	char *code;
+} EphyFontsLanguageInfo;
 
-typedef struct
-{
-	gchar *title;
-	gchar *key;
-	gchar *encoding;
-	LanguageGroup group;
-} EncodingInfo;
-
-/* language encoding groups */
-typedef struct
-{
-	gchar *title;
-	gchar* code;
-} FontsLanguageInfo;
-
-guint			 ephy_langs_get_n_font_languages (void);
-
-const FontsLanguageInfo *ephy_langs_get_font_languages   (void);
-
-void			 language_group_info_free	 (LanguageGroupInfo *info);
-
-void			 encoding_info_free		 (EncodingInfo *info);
+const EphyFontsLanguageInfo 	*ephy_font_languages	(void);
 
+guint				 ephy_font_n_languages	(void);
 
 G_END_DECLS
 
