@@ -31,7 +31,6 @@
 
 static void ephy_topics_selector_class_init (EphyTopicsSelectorClass *klass);
 static void ephy_topics_selector_init (EphyTopicsSelector *editor);
-static void ephy_topics_selector_finalize (GObject *object);
 static void ephy_topics_selector_set_property (GObject *object,
 		                               guint prop_id,
 		                               const GValue *value,
@@ -41,6 +40,8 @@ static void ephy_topics_selector_get_property (GObject *object,
 		                               GValue *value,
 		                               GParamSpec *pspec);
 
+#define EPHY_TOPICS_SELECTOR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOPICS_SELECTOR, EphyTopicsSelectorPrivate))
+
 struct EphyTopicsSelectorPrivate
 {
 	EphyBookmarks *bookmarks;
@@ -99,8 +100,6 @@ ephy_topics_selector_class_init (EphyTopicsSelectorClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = ephy_topics_selector_finalize;
-
 	object_class->set_property = ephy_topics_selector_set_property;
 	object_class->get_property = ephy_topics_selector_get_property;
 
@@ -109,7 +108,7 @@ ephy_topics_selector_class_init (EphyTopicsSelectorClass *klass)
 					 g_param_spec_object ("bookmarks",
 							      "Bookmarks set",
 							      "Bookmarks set",
-							      EPHY_BOOKMARKS_TYPE,
+							      EPHY_TYPE_BOOKMARKS,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_object_class_install_property (object_class,
@@ -118,23 +117,8 @@ ephy_topics_selector_class_init (EphyTopicsSelectorClass *klass)
 							       "Bookmark",
 							       "Bookmark",
 							       G_PARAM_READWRITE));
-}
 
-static void
-ephy_topics_selector_finalize (GObject *object)
-{
-	EphyTopicsSelector *editor;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (EPHY_IS_TOPIC_SELECTOR (object));
-
-	editor = EPHY_TOPICS_SELECTOR (object);
-
-	g_return_if_fail (editor->priv != NULL);
-
-	g_free (editor->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	g_type_class_add_private (object_class, sizeof(EphyTopicsSelectorPrivate));
 }
 
 void
@@ -235,6 +219,7 @@ fill_model (EphyTopicsSelector *editor)
 					    -1);
 		}
 	}
+	ephy_node_thaw (keywords);
 }
 
 static void
@@ -394,7 +379,7 @@ ephy_topics_build_ui (EphyTopicsSelector *editor)
 static void
 ephy_topics_selector_init (EphyTopicsSelector *editor)
 {
-	editor->priv = g_new0 (EphyTopicsSelectorPrivate, 1);
+	editor->priv = EPHY_TOPICS_SELECTOR_GET_PRIVATE (editor);
 
 	editor->priv->bookmark = NULL;
 }
@@ -408,7 +393,7 @@ ephy_topics_selector_new (EphyBookmarks *bookmarks,
 	g_assert (bookmarks != NULL);
 
 	editor = EPHY_TOPICS_SELECTOR (g_object_new
-			(EPHY_TYPE_TOPIC_SELECTOR,
+			(EPHY_TYPE_TOPICS_SELECTOR,
 			 "bookmarks", bookmarks,
 			 "bookmark", bookmark,
 			 NULL));
