@@ -15,15 +15,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-automation.c,v 1.13 2003/09/24 19:15:32 chpe Exp $
+ *  $Id: ephy-automation.c,v 1.17 2003/11/14 17:11:16 marco Exp $
  */
 
 #include "ephy-automation.h"
-#include "ephy-shell.h"
+
 #include "EphyAutomation.h"
+#include "ephy-shell.h"
 #include "ephy-embed.h"
 #include "ephy-window.h"
-#include "session.h"
+#include "ephy-session.h"
 #include "ephy-bookmarks.h"
 #include "ephy-bookmarks-import.h"
 
@@ -31,91 +32,33 @@
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-context.h>
 
-static void
-impl_ephy_automation_add_bookmark (PortableServer_Servant _servant,
-				   const CORBA_char * url,
-				   CORBA_Environment * ev);
-static void
-impl_ephy_automation_import_bookmarks (PortableServer_Servant _servant,
-				       const CORBA_char * filename,
-				       CORBA_Environment * ev);
-static void
-impl_ephy_automation_quit (PortableServer_Servant _servant,
-                           CORBA_Environment * ev);
-static void
-impl_ephy_automation_load_session (PortableServer_Servant _servant,
-				   const CORBA_char * filename,
-				   CORBA_Environment * ev);
-static void
-ephy_automation_class_init (EphyAutomationClass *klass);
-static void
-ephy_automation_init (EphyAutomation *a);
-static void
-ephy_automation_object_finalize (GObject *object);
-static BonoboObject *
-ephy_automation_factory (BonoboGenericFactory *this_factory,
-			 const char *iid,
-			 gpointer user_data);
+static void ephy_automation_class_init (EphyAutomationClass *klass);
 
-static GObjectClass *ephy_automation_parent_class;
-
-#define EPHY_FACTORY_OAFIID "OAFIID:GNOME_Epiphany_Automation_Factory"
-
-static BonoboObject *
-ephy_automation_factory (BonoboGenericFactory *this_factory,
-			 const char *iid,
-			 gpointer user_data)
-{
-        EphyAutomation *a;
-
-        a  = g_object_new (EPHY_TYPE_AUTOMATION, NULL);
-
-        return BONOBO_OBJECT(a);
-}
-
-BonoboGenericFactory *
-ephy_automation_factory_new (void)
-{
-	BonoboGenericFactory *factory;
-
-	factory = bonobo_generic_factory_new (EPHY_FACTORY_OAFIID,
-					      ephy_automation_factory,
-					      NULL);
-	if (factory == NULL)
-	{
-		g_warning ("Could not initialize EphyAutomation factory");
-	}
-
-	return factory;
-}
+static GObjectClass *parent_class = NULL;
 
 static void
 impl_ephy_automation_loadurl (PortableServer_Servant _servant,
-			      const CORBA_char * url,
+			      const CORBA_char *url,
 			      const CORBA_boolean fullscreen,
 			      const CORBA_boolean open_in_existing_tab,
-			      const CORBA_boolean open_in_new_window,
 			      const CORBA_boolean open_in_new_tab,
-			      const CORBA_boolean raise,
-                              CORBA_Environment * ev)
+                              CORBA_Environment *ev)
 {
 	EphyNewTabFlags flags = 0;
 	EphyWindow *window;
-	Session *session;
+	EphySession *session;
 
 	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+	g_return_if_fail (session != NULL);
 
-	/* no window open, let's try to autoresume */
-	if (session_get_windows (session) == NULL)
+	if (ephy_session_autoresume (session) && *url == '\0')
 	{
-		gboolean res;
-		res = session_autoresume (session);
 		/* no need to open the homepage,
-		 * we did already open session windows */
-		if (res && *url == '\0') return;
+		* we did already open session windows */
+		return;
 	}
 
-	window = ephy_shell_get_active_window (ephy_shell);
+	window = ephy_session_get_active_window (session);
 
 	if (open_in_existing_tab && window != NULL)
 	{
@@ -168,52 +111,26 @@ impl_ephy_automation_import_bookmarks (PortableServer_Servant _servant,
 			       filename);
 }
 
-static void
-impl_ephy_automation_quit (PortableServer_Servant _servant,
-                           CORBA_Environment * ev)
-{
-	Session *session;
-
-	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-
-	session_close (session);
-}
-
 static void
 impl_ephy_automation_load_session (PortableServer_Servant _servant,
 				   const CORBA_char * filename,
 				   CORBA_Environment * ev)
 {
-	Session *session;
+	EphySession *session;
 
 	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-	session_load (session, filename);
+	ephy_session_load (session, filename);
 }
 
 static void
 impl_ephy_automation_open_bookmarks_editor (PortableServer_Servant _servant,
 					    CORBA_Environment * ev)
 {
-	ephy_shell_show_bookmarks_editor (ephy_shell, NULL);
-}
+	GtkWidget *editor;
 
-static void
-ephy_automation_class_init (EphyAutomationClass *klass)
-{
-        GObjectClass *object_class = (GObjectClass *) klass;
-        POA_GNOME_EphyAutomation__epv *epv = &klass->epv;
+	editor = ephy_shell_get_bookmarks_editor (ephy_shell);
 
-        ephy_automation_parent_class = g_type_class_peek_parent (klass);
-
-        object_class->finalize = ephy_automation_object_finalize;
-
-        /* connect implementation callbacks */
-        epv->loadurl = impl_ephy_automation_loadurl;
-	epv->addBookmark = impl_ephy_automation_add_bookmark;
-	epv->importBookmarks = impl_ephy_automation_import_bookmarks;
-	epv->quit = impl_ephy_automation_quit;
-	epv->loadSession = impl_ephy_automation_load_session;
-	epv->openBookmarksEditor = impl_ephy_automation_open_bookmarks_editor;
+	gtk_window_present (GTK_WINDOW (editor));
 }
 
 static void
@@ -222,11 +139,18 @@ ephy_automation_init (EphyAutomation *c)
 }
 
 static void
-ephy_automation_object_finalize (GObject *object)
+ephy_automation_class_init (EphyAutomationClass *klass)
 {
-        EphyAutomation *a = EPHY_AUTOMATION (object);
+	POA_GNOME_EphyAutomation__epv *epv = &klass->epv;
 
-        ephy_automation_parent_class->finalize (G_OBJECT (a));
+	parent_class = g_type_class_peek_parent (klass);
+
+	/* connect implementation callbacks */
+	epv->loadurl = impl_ephy_automation_loadurl;
+	epv->addBookmark = impl_ephy_automation_add_bookmark;
+	epv->importBookmarks = impl_ephy_automation_import_bookmarks;
+	epv->loadSession = impl_ephy_automation_load_session;
+	epv->openBookmarksEditor = impl_ephy_automation_open_bookmarks_editor;
 }
 
 BONOBO_TYPE_FUNC_FULL (
