@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history-window.c,v 1.59 2003/10/20 07:55:06 marco Exp $
+ *  $Id: ephy-history-window.c,v 1.63 2003/11/22 14:40:37 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,7 @@
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtkuimanager.h>
 #include <gdk/gdkkeysyms.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
 #include <time.h>
@@ -53,7 +53,7 @@
 #include "toolbar.h"
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
-#include "session.h"
+#include "ephy-session.h"
 #include "ephy-favicon-cache.h"
 #include "eel-gconf-extensions.h"
 
@@ -164,7 +164,7 @@ static GtkActionEntry ephy_history_ui_entries [] = {
 	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
 	  N_("Delete the selected history link"),
 	  G_CALLBACK (cmd_delete) },
-	{ "BookmarkLink", EPHY_STOCK_BOOKMARK_PAGE, N_("Boo_kmark Link..."), "<control>D",
+	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("Boo_kmark Link..."), "<control>D",
 	  N_("Bookmark the selected history link"),
 	  G_CALLBACK (cmd_bookmark_link) },
 	{ "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -353,7 +353,10 @@ get_target_window (EphyHistoryWindow *editor)
 	}
 	else
 	{
-		return GTK_WIDGET (ephy_shell_get_active_window (ephy_shell));
+		EphySession *session;
+
+		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+		return GTK_WIDGET (ephy_session_get_active_window (session));
 	}
 }
 
@@ -612,10 +615,21 @@ ephy_history_window_get_type (void)
 	return ephy_history_window_type;
 }
 
+static void
+ephy_history_window_show (GtkWidget *widget)
+{
+	EphyHistoryWindow *window = EPHY_HISTORY_WINDOW (widget);
+
+	gtk_widget_grab_focus (window->priv->search_entry);
+
+	GTK_WIDGET_CLASS (parent_class)->show (widget);
+}
+
 static void
 ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -625,6 +639,8 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 	object_class->get_property = ephy_history_window_get_property;
 	object_class->dispose  = ephy_history_window_dispose;
 
+	widget_class->show = ephy_history_window_show;
+
 	g_object_class_install_property (object_class,
 					 PROP_HISTORY,
 					 g_param_spec_object ("history",
@@ -695,16 +711,13 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	if (GTK_IS_EDITABLE (focus_widget))
 	{
 		gboolean has_selection;
-		gboolean clipboard_contains_text;
 
 		has_selection = gtk_editable_get_selection_bounds
 			(GTK_EDITABLE (focus_widget), NULL, NULL);
-		clipboard_contains_text = gtk_clipboard_wait_is_text_available
-			(gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
 
 		cut = has_selection;
 		copy = has_selection;
-		paste = clipboard_contains_text;
+		paste = TRUE;
 		select_all = TRUE;
 	}
 	else
