@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-shell.h,v 1.24 2003/12/07 10:23:31 chpe Exp $
  */
 
 #ifndef EPHY_SHELL_H
@@ -36,28 +38,42 @@ G_BEGIN_DECLS
 #define EPHY_IS_SHELL_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_SHELL))
 #define EPHY_SHELL_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_SHELL, EphyShellClass))
 
-/* FIXME: huh? */
-#ifndef EPHY_SHELL_TYPE_DEF
 typedef struct EphyShell EphyShell;
-#define EPHY_SHELL_TYPE_DEF
-#endif
-
 typedef struct EphyShellClass EphyShellClass;
 typedef struct EphyShellPrivate EphyShellPrivate;
 
 extern EphyShell *ephy_shell;
 
+#define EPHY_SHELL_ERROR ephy_shell_error_quark ()
+
+typedef enum
+{
+	EPHY_SHELL_ERROR_MISSING_SERVER,
+	EPHY_SHELL_ERROR_OBJECT_REG_FAILED,
+	EPHY_SHELL_ERROR_FACTORY_REG_FAILED
+} EphyShellError;
+
+typedef enum
+{
+	EPHY_SHELL_STARTUP_TABS			= 1 << 0,
+	EPHY_SHELL_STARTUP_EXISTING_WINDOW	= 1 << 2,
+	EPHY_SHELL_STARTUP_FULLSCREEN		= 1 << 3,
+	EPHY_SHELL_STARTUP_BOOKMARKS_EDITOR	= 1 << 4,
+	EPHY_SHELL_STARTUP_SESSION		= 1 << 5,
+	EPHY_SHELL_STARTUP_IMPORT_BOOKMARKS	= 1 << 6,
+	EPHY_SHELL_STARTUP_ADD_BOOKMARK		= 1 << 7,
+	EPHY_SHELL_STARTUP_SERVER		= 1 << 8
+} EphyShellStartupFlags;
+
 typedef enum
 {
 	/* Page types */
 	EPHY_NEW_TAB_HOME_PAGE		= 1 << 0,
 	EPHY_NEW_TAB_NEW_PAGE		= 1 << 1,
 	EPHY_NEW_TAB_OPEN_PAGE		= 1 << 2,
-	EPHY_NEW_TAB_CLONE_PAGE		= 1 << 3,
 
 	/* Page mode */
 	EPHY_NEW_TAB_FULLSCREEN_MODE	= 1 << 4,
-	EPHY_NEW_TAB_SOURCE_MODE	= 1 << 5,
 
 	/* Tabs */
 	EPHY_NEW_TAB_APPEND_GROUPED	= 1 << 6,
@@ -70,39 +86,58 @@ typedef enum
 
 struct EphyShell
 {
-        EphyEmbedShell parent;
-        EphyShellPrivate *priv;
+	EphyEmbedShell parent;
+
+	/*< private >*/
+	EphyShellPrivate *priv;
 };
 
 struct EphyShellClass
 {
-        EphyEmbedShellClass parent_class;
+	EphyEmbedShellClass parent_class;
 };
 
-GType               ephy_shell_get_type              (void);
+GType		ephy_shell_get_type			(void);
 
-EphyShell	   *ephy_shell_new                   (void);
+GQuark		ephy_shell_error_quark			(void);
 
-EphyWindow	   *ephy_shell_get_active_window     (EphyShell *gs);
+EphyShell      *ephy_shell_new				(void);
 
-EphyTab            *ephy_shell_new_tab	             (EphyShell *shell,
-						      EphyWindow *parent_window,
-						      EphyTab *previous_tab,
-						      const char *url,
-						      EphyNewTabFlags flags);
+gboolean	ephy_shell_startup			(EphyShell *shell,
+							 EphyShellStartupFlags flags,
+							 const char **args,
+							 const char *string_arg,
+							 GError **error);
 
-GObject		   *ephy_shell_get_session	     (EphyShell *gs);
+EphyWindow     *ephy_shell_get_active_window		(EphyShell *shell);
 
-EphyBookmarks      *ephy_shell_get_bookmarks         (EphyShell *gs);
+EphyTab	       *ephy_shell_new_tab			(EphyShell *shell,
+							 EphyWindow *parent_window,
+							 EphyTab *previous_tab,
+							 const char *url,
+							 EphyNewTabFlags flags);
 
-GObject		   *ephy_shell_get_toolbars_model    (EphyShell *gs,
-						      gboolean fullscreen);
+GObject	      *ephy_shell_get_session			(EphyShell *shell);
 
-void		    ephy_shell_show_bookmarks_editor (EphyShell *gs,
-						      GtkWidget *parent);
+EphyBookmarks  *ephy_shell_get_bookmarks		(EphyShell *shell);
 
-void		    ephy_shell_show_history_window   (EphyShell *gs,
-						      GtkWidget *parent);
+GObject	      *ephy_shell_get_toolbars_model		(EphyShell *shell,
+							 gboolean fullscreen);
+
+GObject	       *ephy_shell_get_extensions_manager	(EphyShell *shell);
+
+GtkWidget      *ephy_shell_get_bookmarks_editor 	(EphyShell *shell);
+
+GtkWidget      *ephy_shell_get_history_window		(EphyShell *shell);
+
+GObject        *ephy_shell_get_pdm_dialog		(EphyShell *shell);
+
+GObject        *ephy_shell_get_prefs_dialog		(EphyShell *shell);
+
+GObject	       *ephy_shell_get_print_setup_dialog	(EphyShell *shell);
+
+void		ephy_shell_delete_on_exit		(EphyShell *shell,
+							 const char *path);
 
 G_END_DECLS
 
