@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.59 2003/09/24 13:52:48 marco Exp $
+ *  $Id: ephy-tab.c,v 1.71 2003/12/08 15:21:31 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,16 +27,17 @@
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-embed-factory.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-debug.h"
 #include "ephy-string.h"
 #include "ephy-notebook.h"
 #include "ephy-file-helpers.h"
 #include "ephy-zoom.h"
-#include "session.h"
 #include "ephy-favicon-cache.h"
+#include "ephy-embed-persist.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkbutton.h>
@@ -73,6 +75,7 @@ struct EphyTabPrivate
 	int height;
 	GtkToggleAction *action;
 	float zoom;
+	gboolean setting_zoom;
 	EmbedSecurityLevel security_level;
 	TabNavigationFlags nav_flags;
 };
@@ -107,7 +110,7 @@ static void	ephy_tab_set_icon_address	(EphyTab *tab,
 static void	ephy_tab_set_load_status	(EphyTab *tab,
 						 gboolean status);
 static void	ephy_tab_set_link_message	(EphyTab *tab,
-						 const char *message);
+						 char *message);
 static void	ephy_tab_set_load_percent	(EphyTab *tab,
 						 int percent);
 static void	ephy_tab_update_navigation_flags(EphyTab *tab);
@@ -391,6 +394,29 @@ ephy_tab_finalize (GObject *object)
 	LOG ("EphyTab finalized %p", tab)
 }
 
+static gboolean
+address_has_web_scheme (const char *address)
+{
+	GnomeVFSURI *uri;
+	const char *scheme;
+	gboolean has_web_scheme = FALSE;
+
+	uri = gnome_vfs_uri_new (address);
+	if (uri != NULL)
+	{
+		scheme = gnome_vfs_uri_get_scheme (uri);
+
+		has_web_scheme = (strcmp (scheme, "http") == 0 ||
+				  strcmp (scheme, "https") == 0 ||
+				  strcmp (scheme, "ftp") == 0 ||
+				  strcmp (scheme, "file") == 0);
+
+		gnome_vfs_uri_unref (uri);
+	}
+
+	return has_web_scheme;
+}
+
 /* Public functions */
 
 EphyTab *
@@ -422,12 +448,12 @@ ephy_tab_get_load_status (EphyTab *tab)
 }
 
 static void
-ephy_tab_set_link_message (EphyTab *tab, const char *message)
+ephy_tab_set_link_message (EphyTab *tab, char *message)
 {
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_free (tab->priv->link_message);
-	tab->priv->link_message = g_strdup (message);
+	tab->priv->link_message = message;
 
 	g_object_notify (G_OBJECT (tab), "message");
 }
@@ -560,15 +586,16 @@ ephy_tab_favicon_cb (EphyEmbed *embed,
 
 static void
 ephy_tab_link_message_cb (EphyEmbed *embed,
-			  const char *message,
 			  EphyTab *tab)
 {
-	ephy_tab_set_link_message (tab, message);
+	ephy_tab_set_link_message (tab, ephy_embed_get_link_message (embed));
 }
 
 static void
 ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
+	LOG ("ephy_tab_address_cb tab %p address %s", tab, address)
+
 	if (tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
 	{
 		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
@@ -577,12 +604,75 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
 	ephy_tab_update_navigation_flags (tab);
+
+	/* restore zoom level */
+	if (address_has_web_scheme (address))
+	{
+		EphyHistory *history;
+		EphyNode *host;
+		GValue value = { 0, };
+		float zoom = 1.0, current_zoom;
+
+		history = ephy_embed_shell_get_global_history
+				(EPHY_EMBED_SHELL (ephy_shell));
+		host = ephy_history_get_host (history, address);
+
+		if (host != NULL && ephy_node_get_property
+				     (host, EPHY_NODE_HOST_PROP_ZOOM, &value))
+		{
+			zoom = g_value_get_float (&value);
+			g_value_unset (&value);
+		}
+
+		current_zoom = ephy_embed_zoom_get (embed);
+		if (zoom != current_zoom)
+		{
+			tab->priv->setting_zoom = TRUE;
+			ephy_embed_zoom_set (embed, zoom, FALSE);
+			tab->priv->setting_zoom = FALSE;
+		}
+	}
 }
 
 static void
 ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 {
+	char *address;
+
+	LOG ("ephy_tab_zoom_changed_cb tab %p zoom %f", tab, zoom)
+
 	ephy_tab_set_zoom (tab, zoom);
+
+	if (tab->priv->setting_zoom)
+	{
+		return;
+	}
+
+	address = ephy_embed_get_location (embed, TRUE);
+	if (address_has_web_scheme (address))
+	{
+		EphyHistory *history;
+		EphyNode *host;
+		GValue value = { 0, };
+		history = ephy_embed_shell_get_global_history
+				(EPHY_EMBED_SHELL (ephy_shell));
+		host = ephy_history_get_host (history, address);
+
+		if (host != NULL)
+		{
+			float zoom;
+
+			zoom = ephy_embed_zoom_get (embed);
+
+			g_value_init (&value, G_TYPE_FLOAT);
+			g_value_set_float (&value, zoom);
+			ephy_node_set_property
+				(host, EPHY_NODE_HOST_PROP_ZOOM, &value);
+			g_value_unset (&value);
+		}
+	}
+
+	g_free (address);
 }
 
 static void
@@ -590,7 +680,7 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 {
 	char *title;
 
-	ephy_embed_get_title (embed, &title);
+	title = ephy_embed_get_title (embed);
 
 	ephy_tab_set_title (tab, title);
 
@@ -681,7 +771,7 @@ build_net_state_message (const char *uri, EmbedState flags)
                 }
                 else if (flags & EMBED_STATE_STOP)
                 {
-			msg = _("Done.");
+			msg = ("");
                 }
         }
 
@@ -764,12 +854,6 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		}
 		else if (state & EMBED_STATE_STOP)
 		{
-			/* tab load completed, save in session */
-			Session *s;
-
-			s = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-			session_save (s, SESSION_CRASHED);
-
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, FALSE);
 			ephy_tab_update_navigation_flags (tab);
@@ -788,7 +872,7 @@ ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
 	EphyWindow *window;
 
 	window = ephy_window_new ();
-	ephy_window_set_chrome (window, chromemask);
+	ephy_window_request_chrome (window, chromemask);
 
 	new_tab = ephy_tab_new ();
         ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
@@ -829,38 +913,27 @@ ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 	ephy_window_remove_tab (tab->priv->window, tab);
 }
 
-static gint
-ephy_tab_open_uri_cb (EphyEmbed *embed, const char *uri,
-		      EphyTab *tab)
-{
-	LOG ("ephy_tab_open_uri_cb %s", uri)
-
-	/* FIXME: what is this function supposed to do ? */
-	return FALSE;
-}
-
 static void
 ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 		     EphyTab *tab)
 {
-	GList *tabs = NULL;
+	GtkWidget *notebook;
 	EphyWindow *window;
 	GtkWidget *widget;
-	EmbedChromeMask chromemask;
 
 	tab->priv->width = width;
 	tab->priv->height = height;
 
 	window = tab->priv->window;
-	tabs = ephy_window_get_tabs (window);
+	notebook = ephy_window_get_notebook (window);
 	widget = GTK_WIDGET (embed);
-	chromemask = ephy_window_get_chrome (window);
 
 	/* Do not resize window with multiple tabs.
 	 * Do not resize window already showed because
 	 * it's not possible to calculate a sensible window
 	 * size based on the embed size */
-	if (g_list_length (tabs) == 1 && !tab->priv->visibility)
+	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) == 1 &&
+	    !tab->priv->visibility)
 	{
 		gtk_widget_set_size_request
 			(widget, width, height);
@@ -869,42 +942,22 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 		 * has been resized. It appears to be the only way
 		 * to have the window sized according to embed
 		 * size correctly.
-		 * We dont do it for XUL dialogs because in that case
-		 * a "forced" requisition appear correct.
 		 */
-		if (!(chromemask & EMBED_CHROME_OPENASCHROME))
-		{
-			g_idle_add (let_me_resize_hack, embed);
-		}
+		g_idle_add (let_me_resize_hack, embed);
 	}
-
-	g_list_free (tabs);
 }
 
 static void
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
 {
-	GnomeVFSURI *uri;
-	const char *scheme;
 	EphyWindow *window;
-	gboolean new_tab = FALSE;
+	gboolean new_tab;
 
 	window = ephy_tab_get_window (tab);
 	g_return_if_fail (window != NULL);
 
-	uri = gnome_vfs_uri_new (link_address);
-	if (uri != NULL)
-	{
-		scheme = gnome_vfs_uri_get_scheme (uri);
-
-		new_tab = (strcmp (scheme, "http") == 0 ||
-			   strcmp (scheme, "https") == 0 ||
-	                   strcmp (scheme, "ftp") == 0 ||
-	                   strcmp (scheme, "file") == 0);
-
-		gnome_vfs_uri_unref (uri);
-	}
+	new_tab = address_has_web_scheme (link_address);
 
 	if (new_tab)
 	{
@@ -919,25 +972,71 @@ open_link_in_new_tab (EphyTab *tab,
 	}
 }
 
-static gint
-ephy_tab_dom_mouse_click_cb  (EphyEmbed *embed,
-			      EphyEmbedEvent *event,
-			      EphyTab *tab)
+static void
+save_property_url (EphyEmbed *embed,
+		   EphyEmbedEvent *event,
+		   const char *property,
+		   const char *key)
+{
+	const char *location;
+	const GValue *value;
+	EphyEmbedPersist *persist;
+
+	ephy_embed_event_get_property (event, property, &value);
+	location = g_value_get_string (value);
+
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_flags (persist, 0);
+	ephy_embed_persist_set_persist_key (persist, key);
+	ephy_embed_persist_set_source (persist, location);
+
+	ephy_embed_persist_save (persist);
+
+	g_object_unref (G_OBJECT(persist));
+}
+
+static gboolean
+ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
+			     EphyEmbedEvent *event,
+			     EphyTab *tab)
 {
 	EphyEmbedEventType type;
 	EmbedEventContext context;
 	EphyWindow *window;
+	guint modifier;
+	gboolean handled = TRUE;
+	gboolean with_control, with_shift, is_left_click, is_middle_click;
+	gboolean is_link, is_image, is_middle_clickable;
+	gboolean middle_click_opens;
+
+	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
 	window = ephy_tab_get_window (tab);
 	g_return_val_if_fail (window != NULL, FALSE);
 
-	g_assert (EPHY_IS_EMBED_EVENT(event));
+	type = ephy_embed_event_get_event_type (event);
+	context = ephy_embed_event_get_context (event);
+	modifier = ephy_embed_event_get_modifier (event);
 
-	ephy_embed_event_get_event_type (event, &type);
-	ephy_embed_event_get_context (event, &context);
+	with_control = (modifier & GDK_CONTROL_MASK) != 0;
+	with_shift = (modifier & GDK_SHIFT_MASK) != 0;
+	is_left_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON1);
+	is_middle_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2);
 
-	if (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2
-	    && (context & EMBED_CONTEXT_LINK))
+	middle_click_opens = eel_gconf_get_boolean
+				(CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL);
+
+	is_link = (context & EMBED_CONTEXT_LINK) != 0;
+	is_image = (context & EMBED_CONTEXT_IMAGE) != 0;
+	is_middle_clickable = !((context & EMBED_CONTEXT_LINK)
+				|| (context & EMBED_CONTEXT_INPUT)
+				|| (context & EMBED_CONTEXT_EMAIL_LINK));
+
+	/* ctrl+click or middle click opens the link in new tab */
+	if (is_link && ((is_left_click && with_control) || is_middle_click))
 	{
 		const GValue *value;
 		const char *link_address;
@@ -946,20 +1045,31 @@ ephy_tab_dom_mouse_click_cb  (EphyEmbed *embed,
 		link_address = g_value_get_string (value);
 		open_link_in_new_tab (tab, link_address);
 	}
-	else if (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2 &&
-		 eel_gconf_get_boolean (CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL) &&
-		 !(context & EMBED_CONTEXT_LINK
-		   || context & EMBED_CONTEXT_EMAIL_LINK
-		   || context & EMBED_CONTEXT_INPUT))
+	/* shift+click saves the link target */
+	else if (is_link && is_left_click && with_shift)
+	{
+		save_property_url (embed, event, "link", CONF_STATE_DOWNLOAD_DIR);
+	}
+	/* shift+click saves the non-link image */
+	else if (is_image && is_left_click && with_shift)
+	{
+		save_property_url (embed, event, "image", CONF_STATE_SAVE_IMAGE_DIR);
+	}
+	/* middle click opens the selection url */
+	else if (is_middle_clickable && is_middle_click && middle_click_opens)
 	{
-		/* paste url */
 		gtk_selection_convert (GTK_WIDGET (window),
 				       GDK_SELECTION_PRIMARY,
 				       GDK_SELECTION_TYPE_STRING,
 				       GDK_CURRENT_TIME);
 	}
+	/* we didn't handle the event */
+	else
+	{
+		handled = FALSE;
+	}
 
-	return FALSE;
+	return handled;
 }
 
 static void
@@ -973,15 +1083,11 @@ static void
 ephy_tab_init (EphyTab *tab)
 {
 	GObject *embed, *embed_widget;
-	EphyEmbedSingle *single;
 	EphyFaviconCache *cache;
 	char *id;
 
 	LOG ("EphyTab initialising %p", tab)
 
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
 	tab->priv = EPHY_TAB_GET_PRIVATE (tab);
 
 	tab->priv->window = NULL;
@@ -1000,9 +1106,11 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->security_level = STATE_IS_UNKNOWN;
 	tab->priv->status_message = NULL;
 	tab->priv->zoom = 1.0;
+	tab->priv->setting_zoom = FALSE;
 	tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 
-	tab->priv->embed = ephy_embed_new (G_OBJECT(single));
+	tab->priv->embed = EPHY_EMBED
+		(ephy_embed_factory_new_object ("EphyEmbed"));
 	g_assert (tab->priv->embed != NULL);
 
 	embed = G_OBJECT (tab->priv->embed);
@@ -1029,13 +1137,13 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (embed_widget, "destroy",
 			  G_CALLBACK (ephy_tab_embed_destroy_cb),
 			  tab);
-	g_signal_connect (embed, "ge_link_message",
+	g_signal_connect (embed, "link_message",
 			  G_CALLBACK (ephy_tab_link_message_cb),
 			  tab);
 	g_signal_connect (embed, "ge_location",
 			  G_CALLBACK (ephy_tab_address_cb),
 			  tab);
-	g_signal_connect (embed, "ge_title",
+	g_signal_connect (embed, "title",
 			  G_CALLBACK (ephy_tab_title_cb),
 			  tab);
 	g_signal_connect (embed, "ge_zoom_change",
@@ -1047,16 +1155,13 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (embed, "ge_new_window",
 			  G_CALLBACK (ephy_tab_new_window_cb),
 			  tab);
-	g_signal_connect (embed, "ge_visibility",
+	g_signal_connect (embed, "visibility",
 			  G_CALLBACK (ephy_tab_visibility_cb),
 			  tab);
-	g_signal_connect (embed, "ge_destroy_brsr",
+	g_signal_connect (embed, "destroy_browser",
 			  G_CALLBACK (ephy_tab_destroy_brsr_cb),
 			  tab);
-	g_signal_connect (embed, "ge_open_uri",
-			  G_CALLBACK (ephy_tab_open_uri_cb),
-			  tab);
-	g_signal_connect (embed, "ge_size_to",
+	g_signal_connect (embed, "size_to",
 			  G_CALLBACK (ephy_tab_size_to_cb),
 			  tab);
 	g_signal_connect (embed, "ge_dom_mouse_click",
@@ -1105,17 +1210,17 @@ ephy_tab_update_navigation_flags (EphyTab *tab)
 
 	embed = tab->priv->embed;
 
-	if (ephy_embed_can_go_up (embed) == G_OK)
+	if (ephy_embed_can_go_up (embed))
 	{
 		flags |= TAB_NAV_UP;
 	}
 
-	if (ephy_embed_can_go_back (embed) == G_OK)
+	if (ephy_embed_can_go_back (embed))
 	{
 		flags |= TAB_NAV_BACK;
 	}
 
-	if (ephy_embed_can_go_forward (embed) == G_OK)
+	if (ephy_embed_can_go_forward (embed))
 	{
 		flags |= TAB_NAV_FORWARD;
 	}
@@ -1172,7 +1277,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 		GnomeVFSURI *uri = NULL;
 		char *address;
 
-		ephy_embed_get_location (tab->priv->embed, TRUE, &address);
+		address = ephy_embed_get_location (tab->priv->embed, TRUE);
 
 		if (address)
 		{
