@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbars-model.c,v 1.18 2003/09/01 23:20:14 chpe Exp $
+ *  $Id: ephy-toolbars-model.c,v 1.20 2003/11/10 12:09:56 chpe Exp $
  */
 
 #include "ephy-toolbars-model.h"
@@ -28,7 +28,7 @@
 #include "ephy-string.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 
 static void ephy_toolbars_model_class_init (EphyToolbarsModelClass *klass);
 static void ephy_toolbars_model_init       (EphyToolbarsModel *t);
@@ -323,6 +323,7 @@ static void
 ephy_toolbars_model_set_bookmarks (EphyToolbarsModel *model, EphyBookmarks *bookmarks)
 {
 	EggToolbarsModel *egg_model = EGG_TOOLBARS_MODEL (model);
+	gboolean success = FALSE;
 
 	model->priv->bookmarks = bookmarks;
 	g_object_ref (model->priv->bookmarks);
@@ -331,10 +332,11 @@ ephy_toolbars_model_set_bookmarks (EphyToolbarsModel *model, EphyBookmarks *book
 
 	if (g_file_test (model->priv->xml_file, G_FILE_TEST_EXISTS))
 	{
-		egg_toolbars_model_load (egg_model,
+		success = egg_toolbars_model_load (egg_model,
 					 model->priv->xml_file);
 	}
-	else
+
+	if (success == FALSE)
 	{
 		const char *default_xml;
 
