@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2000, 2001, 2002, 2003 Marco Pesenti Gritti
  *
@@ -15,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.124 2003/10/18 11:29:17 chpe Exp $
+ *  $Id: ephy-window.c,v 1.144 2003/12/08 15:21:31 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,6 +24,7 @@
 #endif
 
 #include "ephy-window.h"
+#include "ephy-command-manager.h"
 #include "ephy-bookmarks-menu.h"
 #include "ephy-favorites-menu.h"
 #include "ephy-state.h"
@@ -42,11 +44,11 @@
 #include "ephy-encoding-menu.h"
 #include "ephy-tabs-menu.h"
 #include "ephy-stock-icons.h"
-#include "session.h"
+#include "ephy-extension.h"
 #include "ephy-favicon-cache.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <gtk/gtk.h>
@@ -56,6 +58,15 @@
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtktoggleaction.h>
 
+static void ephy_window_class_init		(EphyWindowClass *klass);
+static void ephy_window_init			(EphyWindow *gs);
+static void ephy_window_finalize		(GObject *object);
+static void ephy_window_show			(GtkWidget *widget);
+static void ephy_window_notebook_switch_page_cb	(GtkNotebook *notebook,
+						 GtkNotebookPage *page,
+						 guint page_num,
+						 EphyWindow *window);
+
 static GtkActionEntry ephy_menu_entries [] = {
 
 	/* Toplevel */
@@ -72,7 +83,7 @@ static GtkActionEntry ephy_menu_entries [] = {
 	{ "FileNewWindow", GTK_STOCK_NEW, N_("_New Window"), "<control>N",
 	  N_("Open a new window"),
 	  G_CALLBACK (window_cmd_file_new_window) },
-	{ "FileNewTab", EPHY_STOCK_NEW_TAB, N_("New _Tab"), "<control>T",
+	{ "FileNewTab", STOCK_NEW_TAB, N_("New _Tab"), "<control>T",
 	  N_("Open a new tab"),
 	  G_CALLBACK (window_cmd_file_new_tab) },
 	{ "FileOpen", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
@@ -81,10 +92,16 @@ static GtkActionEntry ephy_menu_entries [] = {
 	{ "FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<shift><control>S",
 	  N_("Save the current page"),
 	  G_CALLBACK (window_cmd_file_save_as) },
+	{ "FilePrintSetup", NULL, N_("Print Set_up..."), NULL,
+	  N_("Setup the page settings for printing"),
+	  G_CALLBACK (window_cmd_file_print_setup) },
+	{ "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Pre_view"),"<control><shift>P",
+	  N_("Print preview"),
+	  G_CALLBACK (window_cmd_file_print_preview) },
 	{ "FilePrint", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
 	  N_("Print the current page"),
 	  G_CALLBACK (window_cmd_file_print) },
-	{ "FileSendTo", EPHY_STOCK_SEND_LINK, N_("S_end To..."), NULL,
+	{ "FileSendTo", STOCK_SEND_MAIL, N_("S_end To..."), NULL,
 	  N_("Send a link of the current page"),
 	  G_CALLBACK (window_cmd_file_send_to) },
 	{ "FileCloseWindow", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -92,6 +109,12 @@ static GtkActionEntry ephy_menu_entries [] = {
 	  G_CALLBACK (window_cmd_file_close_window) },
 
 	/* Edit menu */
+	{ "EditUndo", GTK_STOCK_UNDO, N_("_Undo"), "<control>Z",
+	  N_("Undo the last action"),
+	  G_CALLBACK (window_cmd_edit_undo) },
+	{ "EditRedo", GTK_STOCK_REDO, N_("Re_do"), "<shift><control>Z",
+	  N_("Redo the last undone action"),
+	  G_CALLBACK (window_cmd_edit_redo) },
 	{ "EditCut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
 	  N_("Cut the selection"),
 	  G_CALLBACK (window_cmd_edit_cut) },
@@ -142,12 +165,12 @@ static GtkActionEntry ephy_menu_entries [] = {
 	{ "ViewEncoding", NULL, N_("Text _Encoding"), NULL,
 	  N_("Change the text encoding"),
 	  NULL },
-	{ "ViewPageSource", EPHY_STOCK_VIEWSOURCE, N_("_Page Source"), "<control>U",
+	{ "ViewPageSource", STOCK_VIEW_SOURCE, N_("_Page Source"), "<control>U",
 	  N_("View the source code of the page"),
 	  G_CALLBACK (window_cmd_view_page_source) },
 
 	/* Bookmarks menu */
-	{ "FileBookmarkPage", EPHY_STOCK_BOOKMARK_PAGE, N_("_Add Bookmark..."), "<control>D",
+	{ "FileBookmarkPage", STOCK_ADD_BOOKMARK, N_("_Add Bookmark..."), "<control>D",
 	  N_("Add a bookmark for the current page"),
 	  G_CALLBACK (window_cmd_file_bookmark_page) },
 	{ "GoBookmarks", EPHY_STOCK_BOOKMARKS, N_("_Edit Bookmarks"), "<control>B",
@@ -213,7 +236,7 @@ static GtkToggleActionEntry ephy_menu_toggle_entries [] =
 	{ "ViewStatusbar", NULL, N_("St_atusbar"), NULL,
 	  N_("Show or hide statusbar"),
 	  G_CALLBACK (window_cmd_view_statusbar), TRUE },
-	{ "ViewFullscreen", EPHY_STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
+	{ "ViewFullscreen", STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
 	  N_("Browse at full screen"),
 	  G_CALLBACK (window_cmd_view_fullscreen), FALSE}
 };
@@ -223,7 +246,7 @@ static GtkActionEntry ephy_popups_entries [] = {
 	/* Document */
 	{ "SaveBackgroundAs", NULL, N_("_Save Background As..."), NULL,
 	  NULL, G_CALLBACK (popup_cmd_save_background_as) },
-	{ "ContextBookmarkPage", EPHY_STOCK_BOOKMARK_PAGE, N_("Add Boo_kmark..."), "<control>D",
+	{ "ContextBookmarkPage", STOCK_ADD_BOOKMARK, N_("Add Boo_kmark..."), "<control>D",
 	  N_("Add a bookmark for the current page"),
 	  G_CALLBACK (window_cmd_file_bookmark_page) },
 
@@ -242,9 +265,9 @@ static GtkActionEntry ephy_popups_entries [] = {
 	  NULL, G_CALLBACK (popup_cmd_link_in_new_window) },
 	{ "OpenLinkInNewTab", NULL, N_("Open Link in New _Tab"),
 	  NULL, NULL, G_CALLBACK (popup_cmd_link_in_new_tab) },
-	{ "DownloadLink", GTK_STOCK_SAVE, N_("_Download Link..."), NULL,
+	{ "DownloadLink", GTK_STOCK_SAVE, N_("_Download Link"), NULL,
 	  NULL, G_CALLBACK (popup_cmd_download_link) },
-	{ "BookmarkLink", EPHY_STOCK_BOOKMARK_PAGE, N_("_Bookmark Link..."),
+	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("_Bookmark Link..."),
 	  NULL, NULL, G_CALLBACK (popup_cmd_bookmark_link) },
 	{ "CopyLinkAddress", NULL, N_("_Copy Link Address"), NULL,
 	  NULL, G_CALLBACK (popup_cmd_copy_link_address) },
@@ -265,6 +288,9 @@ static GtkActionEntry ephy_popups_entries [] = {
 };
 static guint ephy_popups_n_entries = G_N_ELEMENTS (ephy_popups_entries);
 
+#define CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME  "/apps/epiphany/lockdown/disable_javascript_chrome"
+#define CONF_LOCKDOWN_HIDE_MENUBAR "/apps/epiphany/lockdown/hide_menubar"
+
 #define EPHY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_WINDOW, EphyWindowPrivate))
 
 struct EphyWindowPrivate
@@ -285,26 +311,20 @@ struct EphyWindowPrivate
 	EphyTab *active_tab;
 	EphyDialog *find_dialog;
 	EphyDialog *print_dialog;
-	EmbedChromeMask chrome_mask;
+	EmbedChromeMask requested_chrome_mask;
+	EmbedChromeMask actual_chrome_mask;
 	gboolean closing;
 	gboolean is_fullscreen;
+	gboolean is_ppview;
 	gboolean has_size;
 	guint num_tabs;
-};
 
-static void
-ephy_window_class_init (EphyWindowClass *klass);
-static void
-ephy_window_init (EphyWindow *gs);
-static void
-ephy_window_finalize (GObject *object);
-static void
-ephy_window_show (GtkWidget *widget);
-static void
-ephy_window_notebook_switch_page_cb (GtkNotebook *notebook,
-				     GtkNotebookPage *page,
-				     guint page_num,
-				     EphyWindow *window);
+	guint disable_js_chrome_notifier_id;
+	guint show_toolbars_notifier_id;
+	guint show_bookmarks_bar_notifier_id;
+	guint show_statusbar_notifier_id;
+	guint hide_menubar_notifier_id;
+};
 
 enum
 {
@@ -341,17 +361,6 @@ ephy_window_get_type (void)
         return ephy_window_type;
 }
 
-static void
-remove_from_session (EphyWindow *window)
-{
-	Session *session;
-
-	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-	g_return_if_fail (session != NULL);
-
-	session_remove_window (session, window);
-}
-
 static void
 ephy_window_destroy (GtkObject *gtkobject)
 {
@@ -361,9 +370,13 @@ ephy_window_destroy (GtkObject *gtkobject)
 
 	if (window->priv->closing == FALSE)
 	{
+		EphyExtension *manager;
+
 		window->priv->closing = TRUE;
 
-		remove_from_session (window);
+		/* Let the extensions detach themselves from the window */
+		manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
+		ephy_extension_detach_window (manager, window);
 	}
 
 	if (window->priv->exit_fullscreen_popup)
@@ -429,10 +442,11 @@ exit_fullscreen_button_clicked_cb (GtkWidget *button, EphyWindow *window)
 }
 
 static void
-update_chromes_visibility (EphyWindow *window, EmbedChromeMask flags)
+update_chromes_visibility (EphyWindow *window)
 {
 	GtkWidget *menubar;
 	gboolean fullscreen;
+	EmbedChromeMask flags = window->priv->actual_chrome_mask;
 
 	fullscreen = window->priv->is_fullscreen;
 
@@ -480,6 +494,60 @@ update_chromes_visibility (EphyWindow *window, EmbedChromeMask flags)
 	}
 }
 
+static void
+update_chrome(EphyWindow *window)
+{
+	EmbedChromeMask chrome_mask;
+
+	if (window->priv->is_ppview)
+	{
+		window->priv->actual_chrome_mask = EMBED_CHROME_PPVIEWTOOLBARON;
+	}
+	else
+	{
+		if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME))
+		{
+			chrome_mask = EMBED_CHROME_DEFAULT;
+		}
+		else
+		{
+			chrome_mask = window->priv->requested_chrome_mask;
+		}
+
+		if ((chrome_mask & EMBED_CHROME_BOOKMARKSBAR_DEFAULT) ||
+		    (chrome_mask & EMBED_CHROME_DEFAULT))
+		{
+			/* keep only not layout flags */
+			chrome_mask &= (EMBED_CHROME_WINDOWRAISED |
+					EMBED_CHROME_WINDOWLOWERED |
+					EMBED_CHROME_CENTERSCREEN |
+					EMBED_CHROME_OPENASDIALOG |
+					EMBED_CHROME_OPENASPOPUP);
+
+			/* Load defaults */
+			if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS))
+			{
+				chrome_mask |= EMBED_CHROME_TOOLBARON;
+			}
+			if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR))
+			{
+				chrome_mask |= EMBED_CHROME_BOOKMARKSBARON;
+			}
+			if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
+			{
+				chrome_mask |= EMBED_CHROME_STATUSBARON;
+			}
+			if (!eel_gconf_get_boolean (CONF_LOCKDOWN_HIDE_MENUBAR))
+			{
+				chrome_mask |= EMBED_CHROME_MENUBARON;
+			}
+		}
+		window->priv->actual_chrome_mask = chrome_mask;
+	}
+
+	update_chromes_visibility (window);
+}
+
 static void
 ephy_window_fullscreen (EphyWindow *window)
 {
@@ -517,7 +585,7 @@ ephy_window_fullscreen (EphyWindow *window)
                           "size-changed", G_CALLBACK (size_changed_cb),
 			  popup);
 
-	update_chromes_visibility (window, window->priv->chrome_mask);
+	update_chromes_visibility (window);
 }
 
 static void
@@ -532,7 +600,7 @@ ephy_window_unfullscreen (EphyWindow *window)
 	gtk_widget_destroy (window->priv->exit_fullscreen_popup);
 	window->priv->exit_fullscreen_popup = NULL;
 
-	update_chromes_visibility (window, window->priv->chrome_mask);
+	update_chromes_visibility (window);
 }
 
 static gboolean
@@ -567,6 +635,102 @@ ephy_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, EphyW
 	return FALSE;
 }
 
+
+static void
+update_edit_actions_sensitivity (EphyWindow *window)
+{
+	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
+	GtkActionGroup *action_group;
+	GtkAction *action;
+	gboolean can_copy, can_cut, can_undo, can_redo, can_paste;
+
+	if (GTK_IS_EDITABLE (widget))
+	{
+		gboolean has_selection;
+
+		has_selection = gtk_editable_get_selection_bounds
+			(GTK_EDITABLE (widget), NULL, NULL);
+
+		can_copy = has_selection;
+		can_cut = has_selection;
+		can_paste = TRUE;
+		can_undo = FALSE;
+		can_redo = FALSE;
+	}
+	else
+	{
+		EphyEmbed *embed;
+
+		embed = ephy_window_get_active_embed (window);
+		g_return_if_fail (embed != NULL);
+
+		can_copy = ephy_command_manager_can_do_command
+				(EPHY_COMMAND_MANAGER (embed), "cmd_copy");
+		can_cut = ephy_command_manager_can_do_command
+				(EPHY_COMMAND_MANAGER (embed), "cmd_cut");
+		can_paste = ephy_command_manager_can_do_command
+				(EPHY_COMMAND_MANAGER (embed), "cmd_paste");
+		can_undo = ephy_command_manager_can_do_command
+				(EPHY_COMMAND_MANAGER (embed), "cmd_undo");
+		can_redo = ephy_command_manager_can_do_command
+				(EPHY_COMMAND_MANAGER (embed), "cmd_redo");
+	}
+
+	action_group = window->priv->action_group;
+
+	action = gtk_action_group_get_action (action_group, "EditCopy");
+	g_object_set (action, "sensitive", can_copy, NULL);
+	action = gtk_action_group_get_action (action_group, "EditCut");
+	g_object_set (action, "sensitive", can_cut, NULL);
+	action = gtk_action_group_get_action (action_group, "EditPaste");
+	g_object_set (action, "sensitive", can_paste, NULL);
+	action = gtk_action_group_get_action (action_group, "EditUndo");
+	g_object_set (action, "sensitive", can_undo, NULL);
+	action = gtk_action_group_get_action (action_group, "EditRedo");
+	g_object_set (action, "sensitive", can_redo, NULL);
+}
+
+static void
+edit_menu_show_cb (GtkWidget *menu,
+		   EphyWindow *window)
+{
+	update_edit_actions_sensitivity (window);
+}
+
+static void
+edit_menu_hide_cb (GtkWidget *menu,
+		   EphyWindow *window)
+{
+	GtkActionGroup *action_group;
+	GtkAction *action;
+
+	action_group = window->priv->action_group;
+
+	action = gtk_action_group_get_action (action_group, "EditCopy");
+	g_object_set (action, "sensitive", TRUE, NULL);
+	action = gtk_action_group_get_action (action_group, "EditCut");
+	g_object_set (action, "sensitive", TRUE, NULL);
+	action = gtk_action_group_get_action (action_group, "EditUndo");
+	g_object_set (action, "sensitive", TRUE, NULL);
+	action = gtk_action_group_get_action (action_group, "EditRedo");
+	g_object_set (action, "sensitive", TRUE, NULL);
+}
+
+static void
+init_menu_updaters (EphyWindow *window)
+{
+	GtkWidget *edit_menu_item, *edit_menu;
+
+	edit_menu_item = gtk_ui_manager_get_widget
+		(GTK_UI_MANAGER (window->ui_merge), "/menubar/EditMenu");
+	edit_menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (edit_menu_item));
+
+	g_signal_connect (edit_menu, "show",
+			  G_CALLBACK (edit_menu_show_cb), window);
+	g_signal_connect (edit_menu, "hide",
+			  G_CALLBACK (edit_menu_hide_cb), window);
+}
+
 static void
 setup_window (EphyWindow *window)
 {
@@ -643,6 +807,8 @@ setup_window (EphyWindow *window)
 
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (window->ui_merge));
 
+	init_menu_updaters (window);
+
 	window->priv->toolbar = toolbar_new (window);
 	gtk_widget_show (GTK_WIDGET (window->priv->toolbar));
 	gtk_box_pack_end (GTK_BOX (window->priv->menu_dock),
@@ -773,11 +939,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 
 	embed = ephy_tab_get_embed (tab);
 
-	if (ephy_embed_get_security_level (embed, &level, &description) != G_OK)
-	{
-		level = STATE_IS_UNKNOWN;
-		description = NULL;
-	}
+	ephy_embed_get_security_level (embed, &level, &description);
 
 	if (level != ephy_tab_get_security_level (tab))
 	{
@@ -951,17 +1113,6 @@ popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 	*push_in = TRUE;
 }
 
-static void
-popup_destroy_cb (GtkWidget *widget, EphyWindow *window)
-{
-	EphyEmbedEvent *event;
-
-	event = EPHY_EMBED_EVENT (g_object_get_data (G_OBJECT (window), "context_event"));
-	g_object_set_data (G_OBJECT (window), "context_event", NULL);
-
-	g_object_unref (event);
-}
-
 static void
 show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 {
@@ -969,14 +1120,13 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	GtkAction *action;
 	EmbedEventContext context;
 	const char *popup;
-	char *path;
 	const GValue *value;
 	gboolean framed, has_background;
 	GtkWidget *widget;
 	EphyEmbedEventType type;
 
 	/* Do not show the menu in print preview mode */
-	if (window->priv->chrome_mask & EMBED_CHROME_PPVIEWTOOLBARON)
+	if (window->priv->is_ppview)
 	{
 		return;
 	}
@@ -986,29 +1136,32 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 
 	has_background = ephy_embed_event_has_property (event, "background_image");
 
-	ephy_embed_event_get_context (event, &context);
+	context = ephy_embed_event_get_context (event);
 
 	if ((context & EMBED_CONTEXT_LINK) &&
 	    (context & EMBED_CONTEXT_IMAGE))
 	{
-		popup = "EphyImageLinkPopup";
+		popup = "/EphyImageLinkPopup";
 	}
 	else if (context & EMBED_CONTEXT_LINK)
 	{
-		popup = "EphyLinkPopup";
+		popup = "/EphyLinkPopup";
 	}
 	else if (context & EMBED_CONTEXT_IMAGE)
 	{
-		popup = "EphyImagePopup";
+		popup = "/EphyImagePopup";
 	}
-/*	else if (context & EMBED_CONTEXT_INPUT)
+#if MOZILLA_SNAPSHOT > 12
+	else if (context & EMBED_CONTEXT_INPUT)
 	{
-		popup = "EphyInputPopup";
-	}*/
+		update_edit_actions_sensitivity (window);
+		popup = "/EphyInputPopup";
+	}
+#endif
 	else
 	{
-		popup = framed ? "EphyFramedDocumentPopup" :
-				 "EphyDocumentPopup";
+		popup = framed ? "/EphyFramedDocumentPopup" :
+				 "/EphyDocumentPopup";
 	}
 
 	action_group = window->priv->popups_action_group;
@@ -1016,19 +1169,15 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	g_object_set (action, "sensitive", has_background,
 			      "visible", has_background, NULL);
 
-	path = g_strconcat ("/", popup, NULL);
 	widget = gtk_ui_manager_get_widget (GTK_UI_MANAGER (window->ui_merge),
-				            path);
-	g_free (path);
-
+				            popup);
 	g_return_if_fail (widget != NULL);
 
-	g_object_ref (event);
-	g_object_set_data (G_OBJECT (window), "context_event", event);
-	g_signal_connect (widget, "destroy",
-			  G_CALLBACK (popup_destroy_cb), window);
+	g_object_set_data_full (G_OBJECT (window), "context_event",
+				g_object_ref (event),
+				(GDestroyNotify)g_object_unref);
 
-	ephy_embed_event_get_event_type (event, &type);
+	type = ephy_embed_event_get_event_type (event);
 	if (type == EPHY_EMBED_EVENT_KEY)
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
@@ -1043,7 +1192,7 @@ show_embed_popup (EphyWindow *window, EphyTab *tab, EphyEmbedEvent *event)
 	}
 }
 
-static gint
+static gboolean
 tab_context_menu_cb (EphyEmbed *embed,
 		     EphyEmbedEvent *event,
 		     EphyWindow *window)
@@ -1052,7 +1201,7 @@ tab_context_menu_cb (EphyEmbed *embed,
 
 	g_return_val_if_fail (EPHY_IS_WINDOW (window), FALSE);
 	g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
-	g_assert (EPHY_IS_EMBED_EVENT(event));
+	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
 	tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "EphyTab"));
 	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
@@ -1366,26 +1515,91 @@ ephy_window_class_init (EphyWindowClass *klass)
 	g_type_class_add_private (object_class, sizeof(EphyWindowPrivate));
 }
 
+static void
+ensure_default_icon (void)
+{
+	static gboolean ephy_has_default_icon = FALSE;
+	GtkIconTheme *icon_theme;
+	GtkIconInfo *icon_info;
+	const char *icon_file;
+
+	if (ephy_has_default_icon) return;
+
+	/* FIXME listen on icon changes */
+	/* FIXME MultiHead: icon theme is per-display, not global */
+	icon_theme = gtk_icon_theme_get_default ();
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme, "web-browser", -1, 0);
+
+	if (icon_info)
+	{
+
+		icon_file = gtk_icon_info_get_filename (icon_info);
+		if (icon_file)
+		{
+			gtk_window_set_default_icon_from_file (icon_file, NULL);
+		}
+
+		gtk_icon_info_free (icon_info);
+	}
+	else
+	{
+		g_warning ("Web browser gnome icon not found");
+	}
+
+	ephy_has_default_icon = TRUE;
+}
+
+static void
+update_layout_toggles (EphyWindow *window)
+{
+	GtkActionGroup *action_group = GTK_ACTION_GROUP (window->priv->action_group);
+	GtkAction *action;
+
+	action = gtk_action_group_get_action (action_group, "ViewToolbar");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS));
+
+	action = gtk_action_group_get_action (action_group, "ViewBookmarksBar");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR));
+
+	action = gtk_action_group_get_action (action_group, "ViewStatusbar");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+				      eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR));
+}
+
+static void
+chrome_notifier (GConfClient *client,
+		 guint cnxn_id,
+		 GConfEntry *entry,
+		 EphyWindow *window)
+{
+	update_chrome (window);
+	update_layout_toggles(window);
+}
+
 static void
 ephy_window_init (EphyWindow *window)
 {
-	Session *session;
+	EphyExtension *manager;
 
 	LOG ("EphyWindow initialising %p", window)
 
-	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-
 	window->priv = EPHY_WINDOW_GET_PRIVATE (window);
 
 	window->priv->active_tab = NULL;
-	window->priv->chrome_mask = 0;
+	window->priv->requested_chrome_mask = EMBED_CHROME_DEFAULT;
+	window->priv->actual_chrome_mask = 0;
 	window->priv->closing = FALSE;
 	window->priv->ppview_toolbar = NULL;
 	window->priv->exit_fullscreen_popup = NULL;
 	window->priv->num_tabs = 0;
 	window->priv->is_fullscreen = FALSE;
+	window->priv->is_ppview = FALSE;
 	window->priv->has_size = FALSE;
 
+	ensure_default_icon ();
+
 	/* Setup the window and connect verbs */
 	setup_window (window);
 
@@ -1409,25 +1623,29 @@ ephy_window_init (EphyWindow *window)
 	window->priv->enc_menu = ephy_encoding_menu_new (window);
 	window->priv->bmk_menu = ephy_bookmarks_menu_new (window);
 
-	/* Once window is fully created, add it to the session list*/
-	session_add_window (session, window);
-}
+	window->priv->disable_js_chrome_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_JAVASCRIPT_CHROME,
+		 (GConfClientNotifyFunc)chrome_notifier, window);
 
-static void
-save_window_chrome (EphyWindow *window)
-{
-	EmbedChromeMask flags = window->priv->chrome_mask;
+	window->priv->show_toolbars_notifier_id = eel_gconf_notification_add
+		(CONF_WINDOWS_SHOW_TOOLBARS,
+		 (GConfClientNotifyFunc)chrome_notifier, window);
 
-	if (!(flags & EMBED_CHROME_OPENASPOPUP) &&
-	    !(flags & EMBED_CHROME_PPVIEWTOOLBARON))
-	{
-		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR,
-				       flags & EMBED_CHROME_BOOKMARKSBARON);
-		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS,
-				       flags & EMBED_CHROME_TOOLBARON);
-		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR,
-				       flags & EMBED_CHROME_STATUSBARON);
-	}
+	window->priv->show_bookmarks_bar_notifier_id = eel_gconf_notification_add
+		(CONF_WINDOWS_SHOW_BOOKMARKS_BAR,
+		 (GConfClientNotifyFunc)chrome_notifier, window);
+
+	window->priv->show_statusbar_notifier_id = eel_gconf_notification_add
+		(CONF_WINDOWS_SHOW_STATUSBAR,
+		 (GConfClientNotifyFunc)chrome_notifier, window);
+
+	window->priv->hide_menubar_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_HIDE_MENUBAR,
+		 (GConfClientNotifyFunc)chrome_notifier, window);
+
+	/* Once the window is fully created let the extensions attach to it */
+	manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
+	ephy_extension_attach_window (manager, window);
 }
 
 static void
@@ -1435,6 +1653,12 @@ ephy_window_finalize (GObject *object)
 {
         EphyWindow *window = EPHY_WINDOW (object);
 
+	eel_gconf_notification_remove (window->priv->disable_js_chrome_notifier_id);
+	eel_gconf_notification_remove (window->priv->show_toolbars_notifier_id);
+	eel_gconf_notification_remove (window->priv->show_bookmarks_bar_notifier_id);
+	eel_gconf_notification_remove (window->priv->show_statusbar_notifier_id);
+	eel_gconf_notification_remove (window->priv->hide_menubar_notifier_id);
+
 	if (window->priv->find_dialog)
 	{
 		g_object_unref (G_OBJECT (window->priv->find_dialog));
@@ -1471,82 +1695,20 @@ ephy_window_new (void)
 	return EPHY_WINDOW (g_object_new (EPHY_TYPE_WINDOW, NULL));
 }
 
-EmbedChromeMask
-ephy_window_get_chrome (EphyWindow *window)
+void
+ephy_window_request_chrome (EphyWindow *window,
+			    EmbedChromeMask flags)
 {
-	return window->priv->chrome_mask;
-}
-
-static void
-translate_default_chrome (EmbedChromeMask *chrome_mask)
-{
-	gboolean bbar;
-
-	bbar = (*chrome_mask & EMBED_CHROME_BOOKMARKSBAR_DEFAULT) ||
-	       (*chrome_mask & EMBED_CHROME_DEFAULT);
-
-	/* keep only not layout flags */
-	*chrome_mask &= (EMBED_CHROME_WINDOWRAISED |
-			 EMBED_CHROME_WINDOWLOWERED |
-			 EMBED_CHROME_CENTERSCREEN |
-			 EMBED_CHROME_OPENASDIALOG |
-			 EMBED_CHROME_OPENASCHROME |
-			 EMBED_CHROME_OPENASPOPUP);
-
-	/* Load defaults */
-	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
-	{
-		*chrome_mask |= EMBED_CHROME_STATUSBARON;
-	}
-	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS))
-	{
-		*chrome_mask |= EMBED_CHROME_TOOLBARON;
-	}
-	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR) && bbar)
-	{
-		*chrome_mask |= EMBED_CHROME_BOOKMARKSBARON;
-	}
-
-	*chrome_mask |= EMBED_CHROME_MENUBARON;
-}
-
-static void
-update_layout_toggles (EphyWindow *window)
-{
-	GtkActionGroup *action_group = GTK_ACTION_GROUP (window->priv->action_group);
-	EmbedChromeMask mask = window->priv->chrome_mask;
-	GtkAction *action;
-
-	action = gtk_action_group_get_action (action_group, "ViewToolbar");
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				      mask & EMBED_CHROME_TOOLBARON);
-
-	action = gtk_action_group_get_action (action_group, "ViewBookmarksBar");
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				      mask & EMBED_CHROME_BOOKMARKSBARON);
-
-	action = gtk_action_group_get_action (action_group, "ViewStatusbar");
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				      mask & EMBED_CHROME_STATUSBARON);
+	window->priv->requested_chrome_mask = flags;
+	update_chrome (window);
 }
 
 void
-ephy_window_set_chrome (EphyWindow *window,
-			EmbedChromeMask flags)
+ephy_window_set_print_preview (EphyWindow *window, gboolean enabled)
 {
-	if ((flags & EMBED_CHROME_DEFAULT) ||
-	    (flags & EMBED_CHROME_BOOKMARKSBAR_DEFAULT))
-	{
-		translate_default_chrome (&flags);
-	}
-
-	update_chromes_visibility (window, flags);
-
-	window->priv->chrome_mask = flags;
-
-	update_layout_toggles (window);
-
-	save_window_chrome (window);
+	window->priv->is_ppview = enabled;
+	update_chrome (window);
+	ephy_notebook_set_show_tabs (EPHY_NOTEBOOK (window->priv->notebook), !enabled);
 }
 
 GtkWidget *
@@ -1659,17 +1821,15 @@ ephy_window_show (GtkWidget *widget)
 {
 	EphyWindow *window = EPHY_WINDOW(widget);
 
-	if (!window->priv->chrome_mask)
-	{
-		ephy_window_set_chrome (window, EMBED_CHROME_DEFAULT);
-	}
+	update_chrome (window);
+	update_layout_toggles(window);
 
 	if (!window->priv->has_size)
 	{
 		gboolean keep_state = TRUE;
 
 		/* Do not keep state of sized popups */
-		if (window->priv->chrome_mask & EMBED_CHROME_OPENASPOPUP)
+		if (window->priv->actual_chrome_mask & EMBED_CHROME_OPENASPOPUP)
 		{
 			EphyTab *tab;
 			int width, height;
@@ -1812,29 +1972,17 @@ ephy_window_find (EphyWindow *window)
 	ephy_dialog_show (window->priv->find_dialog);
 }
 
-static void
-print_dialog_preview_cb (EphyDialog *dialog,
-			 EphyWindow *window)
-{
-	ephy_window_set_chrome (window, EMBED_CHROME_PPVIEWTOOLBARON);
-	ephy_notebook_set_show_tabs (EPHY_NOTEBOOK (window->priv->notebook), FALSE);
-}
-
 void
 ephy_window_print (EphyWindow *window)
 {
-	EphyDialog *dialog;
-	EphyEmbed *embed;
-
 	if (window->priv->print_dialog == NULL)
 	{
+		EphyDialog *dialog;
+		EphyEmbed *embed;
+
 		embed = ephy_window_get_active_embed (window);
-		dialog = print_dialog_new_with_parent
-					(GTK_WIDGET(window), embed, NULL);
-		g_signal_connect (G_OBJECT (dialog),
-				  "preview",
-				  G_CALLBACK (print_dialog_preview_cb),
-				  window);
+		dialog = ephy_print_dialog_new  (GTK_WIDGET (window), embed, FALSE);
+
 		window->priv->print_dialog = dialog;
 		g_object_add_weak_pointer(G_OBJECT (dialog),
 					  (gpointer *) &window->priv->print_dialog);
@@ -1855,7 +2003,7 @@ ephy_window_set_zoom (EphyWindow *window,
 	embed = ephy_window_get_active_embed (window);
         g_return_if_fail (embed != NULL);
 
-	ephy_embed_zoom_get (embed, &current_zoom);
+	current_zoom = ephy_embed_zoom_get (embed);
 
 	if (zoom == ZOOM_IN)
 	{
