@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: toolbar.c,v 1.67 2003/10/14 11:36:37 marco Exp $
+ *  $Id: toolbar.c,v 1.72 2003/11/22 13:52:34 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,7 +29,6 @@
 #include "ephy-location-entry.h"
 #include "ephy-dnd.h"
 #include "ephy-spinner.h"
-#include "ephy-spinner-action.h"
 #include "ephy-location-action.h"
 #include "ephy-favicon-action.h"
 #include "ephy-topic-action.h"
@@ -43,9 +42,10 @@
 #include "ephy-new-bookmark.h"
 #include "ephy-stock-icons.h"
 #include "ephy-toolbars-model.h"
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtktoolbar.h>
 
@@ -79,6 +79,8 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
+#define CONF_LOCKDOWN_DISABLE_ARBITRARY_URL  "/apps/epiphany/lockdown/disable_arbitrary_url"
+
 #define EPHY_TOOLBAR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOOLBAR, ToolbarPrivate))
 
 struct ToolbarPrivate
@@ -91,6 +93,7 @@ struct ToolbarPrivate
 	GtkWidget *spinner;
 	GtkWidget *favicon;
 	GtkWidget *go;
+	guint disable_arbitrary_url_notifier_id;
 };
 
 GType
@@ -122,6 +125,41 @@ toolbar_get_type (void)
 
 }
 
+static void
+update_location_editable (Toolbar *t)
+{
+	GtkActionGroup *action_group;
+	GtkAction *action;
+	gboolean editable;
+
+	editable = !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL);
+
+	/* Restore the real web page address when disabling entry */
+	if (!editable)
+	{
+		EphyEmbed *embed;
+		char *address;
+
+		embed = ephy_window_get_active_embed (t->priv->window);
+		address = ephy_embed_get_location (embed, TRUE);
+		toolbar_set_location (t, address);
+		g_free (address);
+	}
+
+	action_group = t->priv->action_group;
+	action = gtk_action_group_get_action (action_group, "Location");
+	g_object_set (G_OBJECT (action), "editable", editable, NULL);
+}
+
+static void
+arbitrary_url_notifier (GConfClient *client,
+		        guint cnxn_id,
+		        GConfEntry *entry,
+		        Toolbar *t)
+{
+	update_location_editable (t);
+}
+
 static void
 go_location_cb (GtkAction *action, char *location, EphyWindow *window)
 {
@@ -326,13 +364,6 @@ toolbar_setup_actions (Toolbar *t)
 	gtk_action_group_add_action (t->priv->action_group, action);
 	g_object_unref (action);
 
-	action = g_object_new (EPHY_TYPE_SPINNER_ACTION,
-			       "name", "Spinner",
-			       "label", _("Spinner"),
-			       NULL);
-	gtk_action_group_add_action (t->priv->action_group, action);
-	g_object_unref (action);
-
 	/* FIXME: I'm still waiting for the exact term to 
 	 * user here from the docs team.
 	 */
@@ -347,6 +378,7 @@ toolbar_setup_actions (Toolbar *t)
 	g_signal_connect (action, "notify::address",
 			  G_CALLBACK (sync_user_input_cb), t);
 	gtk_action_group_add_action (t->priv->action_group, action);
+	update_location_editable (t);
 	g_object_unref (action);
 
 	action = g_object_new (EPHY_TYPE_ZOOM_ACTION,
@@ -523,6 +555,10 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 			         G_CALLBACK (window_state_event_cb),
 			         t, 0);
 
+	t->priv->disable_arbitrary_url_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_ARBITRARY_URL,
+		 (GConfClientNotifyFunc)arbitrary_url_notifier, t);
+
 	g_object_set (G_OBJECT (t),
 		      "MenuMerge", t->priv->ui_merge,
 		      NULL);
@@ -530,6 +566,36 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 	init_normal_mode (t);
 }
 
+static void
+toolbar_style_sync (GtkToolbar *toolbar,
+		    GtkToolbarStyle style,
+		    GtkWidget *spinner)
+{
+	gboolean small;
+
+	small = (style != GTK_TOOLBAR_BOTH);
+
+	ephy_spinner_set_small_mode (EPHY_SPINNER (spinner), small);
+}
+
+static void
+create_spinner (Toolbar *t)
+{
+	GtkWidget *spinner;
+	GtkToolbar *toolbar;
+
+	spinner = ephy_spinner_new ();
+	gtk_widget_show (spinner);
+	t->priv->spinner = spinner;
+
+	toolbar = egg_editable_toolbar_set_fixed
+		(EGG_EDITABLE_TOOLBAR (t), spinner);
+
+	g_signal_connect (toolbar, "style_changed",
+			  G_CALLBACK (toolbar_style_sync),
+			  spinner);
+}
+
 static void
 toolbar_init (Toolbar *t)
 {
@@ -539,6 +605,8 @@ toolbar_init (Toolbar *t)
 	t->priv->ui_merge = NULL;
 	t->priv->visibility = TRUE;
 	t->priv->updating_address = FALSE;
+
+	create_spinner (t);
 }
 
 static void
@@ -546,11 +614,13 @@ toolbar_finalize (GObject *object)
 {
 	Toolbar *t = EPHY_TOOLBAR (object);
 
-	/* FIXME: why not at the end? */
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	eel_gconf_notification_remove
+		(t->priv->disable_arbitrary_url_notifier_id);
 
 	g_object_unref (t->priv->action_group);
 
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+
 	LOG ("Toolbar finalized")
 }
 
@@ -577,23 +647,13 @@ toolbar_activate_location (Toolbar *t)
 void
 toolbar_spinner_start (Toolbar *t)
 {
-	GtkActionGroup *action_group;
-	GtkAction *action;
-
-	action_group = t->priv->action_group;
-	action = gtk_action_group_get_action (action_group, "Spinner");
-	g_object_set (action, "throbbing", TRUE, NULL);
+	ephy_spinner_start (EPHY_SPINNER (t->priv->spinner));
 }
 
 void
 toolbar_spinner_stop (Toolbar *t)
 {
-	GtkActionGroup *action_group;
-	GtkAction *action;
-
-	action_group = t->priv->action_group;
-	action = gtk_action_group_get_action (action_group, "Spinner");
-	g_object_set (action, "throbbing", FALSE, NULL);
+	ephy_spinner_stop (EPHY_SPINNER (t->priv->spinner));
 }
 
 void
